/**********************************************************
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 *********************************************************/

/**********************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *********************************************************/

#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <math.h>

#ifdef __WIN32__
#include <conio.h>
#include <winsock.h>
#else
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#endif

#include "mp3splt_utils.h"

#if defined(__BEOS__) && !defined (HAS_GETPASS)
//used for proxy (proxy not implemented)
//#warning Faking getpass() !!!
//char *getpass(char *p)
//{
//	char *ret;
//	ret = malloc(30);
//	if (!ret)
//		return NULL;
//	puts(p);
//	fgets(ret,30,stdin);
//	return ret;
//}
#endif
#if defined(__BEOS__) && (IPPROTO_UDP==1)
// net_server has a weird order for IPPROTO_
#else
#define closesocket close
#endif

//used the base64 algorithm - for proxy (proxy not implemented)
//
// Base64 Algorithm: Base64.java v. 1.3.6 by Robert Harder
// Ported and optimized for C by Matteo Trotta
//
//const char alphabet [] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
//char *encode3to4 (unsigned char *source, int srcoffset, int num, char *destination, int destoffset)
//{
//
//    int inbuff=(num>0?(source[srcoffset]<<16):0)|(num>1?(source[srcoffset+1]<<8):0)|(num>2?(source[srcoffset+2]):0);
//    switch(num)
//    {
//	case 3:
//	    destination[destoffset] = alphabet[(inbuff>>18)];
//	    destination[destoffset+1] = alphabet[(inbuff>>12)&0x3f];
//	    destination[destoffset+2] = alphabet[(inbuff>>6)&0x3f];
//	    destination[destoffset+3] = alphabet[(inbuff)&0x3f];
//	    return destination;
//
//	case 2:
//	    destination[destoffset] = alphabet[(inbuff>>18)];
//	    destination[destoffset+1] = alphabet[(inbuff>>12)&0x3f];
//	    destination[destoffset+2] = alphabet[(inbuff>>6)&0x3f];
//	    destination[destoffset+3] = '=';
//	    return destination;
//
//	case 1:
//	    destination[destoffset] = alphabet[(inbuff>>18)];
//	    destination[destoffset+1] = alphabet[(inbuff>>12)&0x3f];
//	    destination[destoffset+2] = '=';
//	    destination[destoffset+3] = '=';
//	    return destination;
//	default:
//	    return destination;
//    }
//}

//used for proxy (proxy not implemented)
//char *b64 (unsigned char *source, int len)
//{
//	char *out;
//	int d, e=0;
//	d = ((len*4/3)+((len%3)>0?4:0));
//	
//	out = malloc(d+1);
//	
//	memset(out, 0x00, d+1);
//	for(d=0;d<(len-2);d+=3,e+=4)
//		out = encode3to4(source, d, 3, out, e);
//	if(d<len)
//		out = encode3to4(source, d, len-d, out, e);
//
//	return out;
//}
// End of Base64 Algorithm

//if we have a string "test", this functions cuts out the two "
//and cleans the string of other unwanted characters
char *get_cue_value(char *in, char *out, int maxlen)
{
  char *ptr_b, *ptr_e;
  ptr_b = strchr(in, '"');
  if (ptr_b==NULL)
    return NULL;
  ptr_e = strchr(ptr_b+1, '"');
  if (ptr_e==NULL)
    return NULL;
  *ptr_e='\0';
  strncpy(out, ptr_b, maxlen);
  cleanstring(out);
  return out;
}

//we put the filenames from the tags
void put_format_filenames(splt_state *state,
			  int tracks,
			  int *error)
{
  int i;
  char temp[128];
  snprintf(temp,128, "%s",state->tags.id[0].artist);
  //if we have the defaults for the output,
  if (state->options.option_output_is_default)
    {
      //we put the real performer in the artist
      for(i = 0; i < tracks;i++)
	{
	  //we put id.artist if no performer
	  if (state->tags.performer[i][0] != '\0')
	    {
	      snprintf(state->tags.id[i].artist,128,
		       "%s",state->tags.performer[i]);
	    }
	  else
	    {
	      snprintf(state->tags.id[i].artist,128,
		       "%s",temp);
	      if (i != 0)
		{
		  snprintf(state->tags.id[i].album,128,
			   "%s",state->tags.id[0].album);
		  snprintf(state->tags.id[i].year,5,
			   "%s",state->tags.id[0].year);
		}
	    }
	  state->tags.id[i].genre = state->tags.id[0].genre;
	}
      snprintf(state->oformat.format_char,1024,"%s",
	       MP3SPLT_DEFAULT_OUTPUT);
    }
  else
    {
      //we put the same artist and genre everywhere
      for(i = 1;i<tracks;i++)
	{
	  snprintf(state->tags.id[i].artist,128,"%s",
		   state->tags.id[0].artist);
	  snprintf(state->tags.id[i].album,128,
		   "%s",state->tags.id[0].album);
	  snprintf(state->tags.id[i].year,5,
		   "%s",state->tags.id[0].year);
	  state->tags.id[i].genre = state->tags.id[0].genre;
	}
    }
  
  char *new_string = strdup(state->oformat.format_char);
  //if strdup fails
  if (new_string == NULL)
    {
      *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
    }
  
  if (*error >= 0)
    {
      int err1;
      //we put the outputted filename
      libmp3splt_checkparse_outformat(state,new_string,
				      &err1);
      if (err1 >= 0)
	{
	  state->split.current_split = 0;
	  do {
	    //get output format filename
	    put_output_filename(state);
	    state->split.current_split++;
	  } while (state->split.current_split<tracks);
	}
    }
}

//err = possible error that we return
//gets the cue splitpoints from a file
//returns the number of tracks found
int get_cue_splitpoints(char *file, splt_state *state,
			int *error)
{
  //default no error
  *error = MP3SPLT_FREEDB_CUE_OK;
  
  state->tags.id[0].genre= (unsigned char) 12;
  
  //our file
  FILE *file_input;
  //the line we read from the file
  char line[1024];
  char *ptr, *dot;
  ptr = dot = NULL;
  //tracks = number of tracks found
  int tracks = -1, time_for_track = MP3SPLT_TRUE;
  //counter, counts if we have at least one INDEX on the file
  int type, performer = MP3SPLT_FALSE,
    title = MP3SPLT_FALSE, counter = 0;
  
  //we open the file
  if (!(file_input=fopen(file, "r")))
    {
      *error = MP3SPLT_CUE_ERROR_CANNOT_OPEN_FILE_READING;
    }
  else
    //if we can open the file
    {
      //put pointer at the beginning of the file
      if (fseek (file_input, 0, SEEK_SET) == 0)
	{
	  //we read the file line by line
	  while (fgets(line, 1024, file_input)!=NULL)
	    {
	      type = MP3SPLT_CUE_NOTHING;
	      
	      //we read strings from file TRACK,TITLE,AUDIO,PERFORMER,
	      //INDEX
	      if ((strstr(line, "TRACK")!=NULL)&&(strstr(line, "AUDIO")!=NULL))
		{
		  type = MP3SPLT_CUE_TRACK;
		}
	      else if (strstr(line, "TITLE")!=NULL)
		{
		  type = MP3SPLT_CUE_TITLE;
		}
	      else if (strstr(line, "PERFORMER")!=NULL)
		{
		  type = MP3SPLT_CUE_PERFORMER;
		}
	      else if ((ptr=strstr(line, "INDEX 01"))!=NULL)
		{
		  type = MP3SPLT_CUE_INDEX;
		}
	      
	      //we clean previous data
	      clean_one_split_data(state,tracks);
	      
	      //we analyse the previous strings found
	      switch (type)
		{
		  //nothing found
		case MP3SPLT_CUE_NOTHING:break;
		  //TRACK and AUDIO found
		case MP3SPLT_CUE_TRACK:
		  if (tracks==-1) 
		    {
		      tracks = 0;
		    }
		  if (time_for_track) 
		    {
		      //if no performer put performer to \0
		      if (!performer)
			{
			  memset(state->tags.performer[tracks-1],'\0',128);
			}
		      else
			{
			  performer = MP3SPLT_FALSE;
			}
		      //if no title, put title to \0
		      if (!title)
			{
			  memset(state->tags.id[tracks-1].title,'\0',128);
			}
		      else
			{
			  title = MP3SPLT_FALSE;
			}
		      tracks++;
		    }
		  else
		    {
		      *error = MP3SPLT_INVALID_CUE_FILE;
		    }
		  time_for_track = MP3SPLT_FALSE;
		  break;
		  //TITLE found
		case MP3SPLT_CUE_TITLE:
		  if (tracks == -1)
		    {
		      if (get_cue_value(line, state->tags.id[0].album, 128)==NULL)
			{
			  *error = MP3SPLT_INVALID_CUE_FILE;
			}
		    }
		  else
		    {
		      if (tracks > 0)
			{
			  if(get_cue_value(line, state->tags.id[tracks-1].title, 127)==NULL)
			    {
			      *error = MP3SPLT_INVALID_CUE_FILE;
			    }
			}
		      title = MP3SPLT_TRUE;
		    }
		  break;
		  //PERFORMER found
		case MP3SPLT_CUE_PERFORMER:	
		  if (tracks == -1)
		    {
		      //we always have one artist in a cue file, we
		      //put the performers if more than one artist
		      if (get_cue_value(line, state->tags.id[0].artist, 127)==NULL)
			{
			  *error = MP3SPLT_INVALID_CUE_FILE;
			}
		    }
		  else
		    {
		      if (tracks>0)
			{
			  if (get_cue_value(line, state->tags.performer[tracks-1], 127)==NULL)
			    {
			      *error = MP3SPLT_INVALID_CUE_FILE;
			    }
			}
		      performer = MP3SPLT_TRUE;
		    }
		  break;
		  //INDEX 01 found
		  //we get the 
		case MP3SPLT_CUE_INDEX:
		  line[strlen(line)-1]='\0';
		  ptr += 9;
		  if ((dot = strchr(ptr, ':'))==NULL)
		    {
		      *error = MP3SPLT_INVALID_CUE_FILE;
		    }
		  else
		    {
		      //we replace : with . for the sscanf
		      ptr[dot-ptr] = ptr[dot-ptr+3] = '.';
		      //we clean the string for unwanted characters
		      cleanstring(ptr);
		      //we convert to hundreths of seconds and put splitpoints
		      //in the state.split->splitpoints[]
		      if (tracks>0)
			{
			  long hundr_seconds = c_hundreths(ptr);
			  if (hundr_seconds==-1)
			    {
			      *error = MP3SPLT_INVALID_CUE_FILE;
			    }
			  state->split.splitpoints[tracks-1] = hundr_seconds;
			  time_for_track = MP3SPLT_TRUE;
			  counter++;
			}
		    }
		  break;
		default:
		  break;
		}
	    }
	}
      else
	{
	  *error = MP3SPLT_ERROR_SEEKING_FILE;
	}
      
      //if we don't find INDEX on the file, error
      if (counter == 0)
	{
	  *error = MP3SPLT_INVALID_CUE_FILE;
	}
      
      //if no time for track,
      if (!time_for_track) 
	{
	  tracks--;
	}
      
      //we put -1 as the end of the splitpoints
      if (tracks>0) 
	{
	  state->split.splitpoints[tracks] = -1;
	}
      
      fclose(file_input);
      
      put_format_filenames(state,tracks,error);
    }
  
  //we return the number of tracks found
  return tracks;
}

//error = possible error that we return
//returns number of tracks
//see freedb file format documentation on freedb.org
int get_cddb_splitpoints (char *file, splt_state *state,
			  int *error)
{
  //default no error
  *error = MP3SPLT_FREEDB_CDDB_OK;
  
  //our file
  FILE *file_input;
  //the line we read
  char line[1024];
  char prev[10];
  //performer_title_split is where we split if we have
  //performer / title on cddb
  char *number = NULL, *c;
  //for the performer
  char *c2;
  //tracks is the number of tracks
  int tracks = 0, i, j;
  
  //we open the file
  if (!(file_input=fopen(file, "r")))
    {
      *error = MP3SPLT_CDDB_ERROR_CANNOT_OPEN_FILE_READING;
    }
  else
    //if we can open the file
    {
      //we go at the beggining of the file
      if(fseek (file_input, 0, SEEK_SET) == 0)
	{
	  //we search for the string "Track frame offset"
	  do {
	    if ((fgets(line, 1024, file_input))==NULL)
	      {
		*error = MP3SPLT_INVALID_CDDB_FILE;
		break;
	      }
	    number = strstr(line, "Track frame offset");
	  } while (number == NULL);
	  
	  memset(prev, 0, 10);
	  
	  //we read the track offsets
	  do {
	    if ((fgets(line, 1024, file_input))==NULL)
	      {
		*error = MP3SPLT_INVALID_CDDB_FILE;
		break;
	      }
	    line[strlen(line)-1] = '\0';
	    if (strstr(line, "Disc length") != NULL)
	      {
		break;
	      }
	    i = 0;
	    while ((isdigit(line[i])==0) && (line[i]!='\0'))
	      {
		i++;
		number = line + i;
	      }
	    if (number == (line + strlen(line))) 
	      {
		break;
	      }
	    else
	      //we put the offsets in the splitpoint table
	      {
		double temp;
		temp = atof(number);
		state->split.splitpoints[tracks++] = (long) temp * 100;
	      }
	  } while (tracks<MP3SPLT_MAXTRACKS);
	  
	  //we check parse output format
	  //setting the number of tracks is important
	  state->split.splitnumber = tracks;
	  
	  //we clean the old data from the state
	  clean_split_data(state,tracks);
	  
	  //if we have Disc length right after
	  //the splitpoints
	  if (strstr(line,"Disc length") != NULL)
	    {
	      number = strstr(line, "Disc length");
	    }
	  else
	    {
	      //we find out "Disc length"
	      do {
		if ((fgets(line, 1024, file_input))==NULL)
		  {
		    *error = MP3SPLT_INVALID_CDDB_FILE;
		    break;
		  }
		number = strstr(line, "Disc length");
	      } while (number == NULL);
	    }
	  
	  //we get the max disc length (in seconds)
	  i = 0;
	  while ((isdigit(line[i])==0) && (line[i]!='\0')) {
	    i++;
	    number = line + i;
	  }
	  
	  //if no error, ?
	  if (*error != MP3SPLT_INVALID_CDDB_FILE)
	    {
	      double temp2;
	      temp2 = atof(number);
	      //we put it in the state
	      state->split.splitpoints[tracks] = (long) temp2 * 100;
	    }
	  else
	    {
	      state->split.splitpoints[tracks] = 0;	
	    }
	  
	  //we convert the points previously found
	  for (i=tracks-1; i>=0; i--) 
	    {
	      state->split.splitpoints[i] -= 
		state->split.splitpoints[0];
	      state->split.splitpoints[i] /= 75;
	    }
	  //end of splitpoints
	  state->split.splitpoints[tracks+1] = -1;
	  
	  j=0;
	  //if we have found the title of the song or not
	  int title = MP3SPLT_FALSE;
	  //if we have performers or not
	  int performer = MP3SPLT_FALSE;
	  do {
	    title = 0;
	    char temp[10];
	    memset(temp, 0, 10);
	    if ((fgets(line, 1024, file_input))==NULL)
	      {
		*error = MP3SPLT_INVALID_CDDB_FILE;
		break;
	      }
	    line[strlen(line)-1] = '\0';
	    if (strlen(line)>0)
	      {
		if (line[strlen(line)-1]=='\r')
		  line[strlen(line)-1]='\0';
	      }
	    //we search for the title of the disc
	    if (j==0)
	      {
		if (strstr(line, "DTITLE")==NULL)
		  {
		    continue;
		  }
	      }
	    else
	      {
		//we search for the title of the tracks
		if (strstr(line, "TTITLE")==NULL)
		  {
		    continue;
		  }
		else
		  {
		    title = MP3SPLT_TRUE;
		  }
	      }
	
	    if ((number=strchr(line, '='))==NULL) 
	      {
		*error = MP3SPLT_INVALID_CDDB_FILE;
		break;
	      }
	    
	    //we read the string after the = or after the /
	    //the / sign is found on the DTITLE=Artist / Disc
	    //we put the string in number variable
	    //used for the filename
	    if (j>0)
	      {
		int len = number-line;
		if (len>10) len = 10;
		strncpy(temp, line, len);
		if ((c = strchr(number, '/'))!=NULL) 
		  {
		    //if found '/' in TITLE, separate artist / title
		    if (title)
		      {
			//we put artist
			performer = MP3SPLT_TRUE;
			strncpy(state->tags.performer[j-1],number+1,
				128);
			c2 = strchr(state->tags.performer[j-1],'/');
			*c2 = '\0';
			c2 = cut_spaces_at_the_end(c2-1);
		      }
		    else
		      {
			snprintf(state->tags.performer[j-1],128,"_");
		      }
		    cleanstring(state->tags.performer[j-1]);
		    number = c-1;
		  }
		number = cleanstring (number);
	      }
	    
	    //we limit number to 512
	    if (strlen(++number)>512) 
	      {
		number[512]='\0';
	      }
	    
	    //if what we read contains Data or Track
	    if ((j>0)&&(strstr(number, "Data")!=NULL) && 
		(strstr(number, "Track")!=NULL)) 
	      {
		state->split.splitpoints[j-1] = 
		  state->split.splitpoints[j];
		tracks -= 1;
	      }
	    else
	      {
		//otherwise, we put the title
		if ((j>0)&&(strcmp(temp, prev)==0))
		  {
		    strncat(state->tags.id[j-1].title,
			    number, 128-strlen(state->tags.id[j-1].title));
		  }
		else
		  {
		    //we treat DTITLE
		    if (j == 0)
		      {
			//we put the artist
			i=0;
			while ((number[i]!='/') && 
			       (number[i]!='\0')&&(i<127)) 
			  {
			    state->tags.id[0].artist[i] = 
			      number[i];
			    i++;
			  }
			state->tags.id[0].artist[i]='\0';
			//we cut the space at the end of the artist
			int k = strlen(state->tags.id[0].artist)-1;
			while (state->tags.id[0].artist[k] == ' ')
			  {
			    state->tags.id[0].artist[k] = '\0';
			    k--;
			  }
			cleanstring(state->tags.id[1].artist);
			//we put the album
			i += 2;
			number = cut_spaces_from_begin(number+i);
			snprintf(state->tags.id[0].album,
				 127,"%s",number);
			cleanstring(state->tags.id[0].album);
		      }
		    else
		      {
			char *t = number;
			//if we don't have = before, it means
			//we have a performer with a  perf/title
			if (*(number-1) != '=')
			  {
			    t = number-1;
			  }
			t = cut_spaces_from_begin(t);
			strncpy(state->tags.id[j-1].title, t, 128);
		      }
		    j++;
		  }
	      }
	    strncpy(prev, temp, 10);
	    
	  } while (j<=tracks);
	  
	  //we search for
	  //YEAR (the year) and ID3 genre
	  state->tags.id[0].genre= (unsigned char) 12;
	  while ((fgets(line, 1024, file_input))!=NULL)
	    {
	      line[strlen(line)-1] = '\0';
	      if (strlen(line)>0)
		{
		  if (line[strlen(line)-1]=='\r') 
		    {
		      line[strlen(line)-1]='\0';
		    }
		}
	      if (strstr(line, "EXTD")==NULL) 
		{
		  continue;
		}
	      else 
		{
		  if ((number=strchr(line, '='))==NULL) 
		    {
		      break;
		    }
		  else 
		    {
		      if ((c=strstr(number, "YEAR"))!=NULL)
			{
			  strncpy(state->tags.id[0].year, c+6, 4);
			}
		      if ((c=strstr(number, "ID3G"))!=NULL) 
			{
			  strncpy(line, c+6, 3);
			  state->tags.id[0].genre= (unsigned char) atoi(line);
			}
		      break;
		    }
		}
	    }
	  
	  //we put the artist
	  j=0;
	  i += 2;
	  
	  fclose(file_input);
	  
	  put_format_filenames(state,tracks,error);
	}
      else
	{
	  *error = MP3SPLT_ERROR_SEEKING_FILE;
	}
    }
  
  return tracks;
}

//we analyse the buffer for the CDs results
int analyse_cd_buffer (char *buf, int size, 
		       splt_state *sp_state,
		       int *error)
{
  char *c, *add;
  int i;
  cd_state *state = sp_state->fdb.cdstate;
  
  //temporary string
  char str_temp[3];
  //temporary character
  char t;
  //the name of the album
  char *album_name = NULL;
  
  //30 is a safe value for preventing buffer exceed
  if (size<30)
    {
      return 0; 
    }
  
  add = buf+size;
  buf = strstr(buf, "cat=");
  if (buf==NULL)
    {
      return 0;
    }
  else {
    do {
      i=0;
      buf += 4;
      
      //-1 = continue with the next buffer
      if ((c=strchr(buf, '&'))==NULL)
	return -1;
      
      if (c==buf)
	continue;
      memset(state->discs[state->foundcd].category, 0x00, 20);
      strncpy(state->discs[state->foundcd].category, buf, c - buf);
      buf=c+4;
      memset(state->discs[state->foundcd].discid, 0x00, 9);
      strncpy(state->discs[state->foundcd].discid, buf, MP3SPLT_DISCIDLEN);
      buf = buf + MP3SPLT_DISCIDLEN + 2;
      
      //-1 = continue with the next buffer
      if ((c=strchr(buf, '<'))==NULL)
	return -1;
      else if (c == buf) {
	if ((buf=strchr(buf, '>'))==NULL)
	  return -1;
	buf++;
	if ((c=strchr(buf, '<'))==NULL)
	  return -1;
	i=-1;
      }
      
      album_name = calloc(2,1);
      //we count the characters for the realloc
      int count = 2;
      //we get the name of the album
      while ( (buf<c) && (buf<add) )
	{
	  t = *buf;
	  buf++;count++;
	  
	  sprintf(str_temp,"%c",t);
	  strcat(album_name, str_temp);
	  album_name = realloc(album_name, count);
	}
      
      //here we have in album_name the name of the current album
      
      //if we had not initialised the 
      //search_results variable, do it now
      if (sp_state->fdb.search_results->number == 0)
	{
	  sp_state->fdb.search_results->results =
	    malloc(2 * sizeof(one_result));
	  sp_state->fdb.search_results->results[0]
	    .name = strdup(album_name);
	  //if strdup fails
	  if (sp_state->fdb.search_results->results[0].name == NULL)
	    {
	      *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
	      return -1;
	    }
	  sp_state->fdb.search_results->results[sp_state->fdb.search_results->number]
	    .revision_number = 0;
	  //unique identifier	
	  sp_state->fdb.search_results->results[sp_state->fdb.search_results->number]
	    .id = 0;
	  sp_state->fdb.search_results->number++;
	}
      else
	{
	  //if its not a revision
	  if (i != -1)
	    {
	      sp_state->fdb.search_results->results = 
		realloc(sp_state->fdb.search_results->results,
			(sp_state->fdb.search_results->number + 1)
			* sizeof(one_result));
	      sp_state->fdb.search_results->results[sp_state->fdb.search_results->number]
		.name = strdup(album_name);
	      //if strdup fails
	      if (sp_state->fdb.search_results->results[sp_state->fdb.search_results->number]
		  .name == NULL)
		{
		  *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
		  return -1;
		}
	      sp_state->fdb.search_results->results[sp_state->fdb.search_results->number]
		.revision_number = 0;
	      sp_state->fdb.search_results->results[sp_state->fdb.search_results->number]
		.id = (sp_state->fdb.search_results->results[sp_state->fdb.search_results->number - 1]
		       .id + sp_state->fdb.search_results->results[sp_state->fdb.search_results->number - 1]
		       .revision_number + 1);
	      sp_state->fdb.search_results->number++;
	    }
	  else
	    //if it's a revision
	    {
	      //if it's the first revision
	      if (sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1]
		  .revision_number == 0)
		{
		  sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1].revisions =
		    malloc(sizeof(int));
		  
		  sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1].revisions[0]
		    = atoi(album_name);
		  sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1].revision_number++;
		}
	      else
		//if it's not the first revision
		{
		  sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1].revisions =
		    realloc(sp_state->fdb.search_results->results
			    [sp_state->fdb.search_results->number-1].revisions,
			    (sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1]
			     .revision_number + 1)
			    * sizeof(int));
		  
		  sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1].
		    revisions[sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1].revision_number]
		    = atoi(album_name);
		  sp_state->fdb.search_results->results[sp_state->fdb.search_results->number-1].revision_number++;
		}
	    }
	}
      
      //we free the local variable
      //for the album name
      if (album_name != NULL)
	{
	  free(album_name);
	}
      
      state->foundcd++;
      
    } while ((buf<(add-30)) && 
	     (state->foundcd<MP3SPLT_MAXCD) && 
	     ((buf=strstr(buf, "cat="))!=NULL));
    
    return state->foundcd;
  }
}

//char *login (char *s)
//{
//	char *pass, junk[130];
//	fprintf (stdout, "Username: ");
//	fgets(junk, 128, stdin);
//	junk[strlen(junk)-1]='\0';
//	pass = getpass("Password: ");
//	sprintf (s, "%s:%s", junk, pass);
//	memset (pass, 0x00, strlen(pass));
//	free(pass);
//	return s;
//}

struct addr useproxy(FILE *in, struct addr dest)
{

	char line[270];
	//char *ptr;
	
	dest.proxy=0;
	memset(dest.hostname, 0, 256);
	memset(line, 0, 270);
	
	if (in != NULL) {
	  /*
	    //proxy NOT IMPLEMENTED YET
	    
		fseek(in, 0, SEEK_SET);

		if (fgets(line, 266, in)!=NULL) {
			if (strstr(line, "PROXYADDR")!=NULL) {
				line[strlen(line)-1]='\0';
				if ((ptr = strchr(line, '='))!=NULL) {
					ptr++;
					strncpy(dest.hostname, ptr, 255);
				}

				if (fgets(line, 266, in)!=NULL) {
					if (strstr(line, "PROXYPORT")!=NULL) {
						line[strlen(line)-1]='\0';
						if ((ptr = strchr(line, '='))!=NULL) {
							ptr++;
							dest.port = atoi (ptr);
							dest.proxy=1;
						}
						fprintf (stderr, "Using Proxy: %s on Port %d\n", dest.hostname, dest.port);
						if (fgets(line, 266, in)!=NULL) {
							if (strstr(line, "PROXYAUTH")!=NULL) {
								line[strlen(line)-1]='\0';
								if ((ptr = strchr(line, '='))!=NULL) {
									ptr++;
									if (ptr[0]=='1') {
										if (fgets(line, 266, in)!=NULL) {
											dest.auth = malloc(strlen(line)+1);
											if (dest.auth==NULL) {
												perror("malloc");
												exit(1);
											}
											memset(dest.auth, 0x0, strlen(line)+1);
											strncpy(dest.auth, line, strlen(line));
										}
										else {
											login(line);
											dest.auth = b64(line, strlen(line));
											memset(line, 0x00, strlen(line));
										}
									}
								}
							}
						}
						else dest.auth = NULL;
					}
				}
			}
		}
	  */
	}
	
	if (!dest.proxy) {
	  strncpy(dest.hostname, MP3SPLT_FREEDB_SITE, 255);
	  dest.port = MP3SPLT_PORT1;
	}
	
	return dest;
}

//search the freedb according to "search"
//returns possible errors
//we have possible errors in result
int process_freedb_search(splt_state *state,
			  char *search)
{
  //possible error that we will return
  int err = MP3SPLT_FREEDB_OK;
  //socket and internet structures
  struct sockaddr_in host;
  struct hostent *h;
  struct addr dest;
  //e is used for the end of the buffer
  //c is used for the buffer read
  char *c, *e=NULL;
  int i, tot=0;
  //the message delivered to the server
  char *message;
  //the buffer that we are using to read incoming transmission
  char buffer[MP3SPLT_BUFFERSIZE];
  
  //fd = socket identifier
#ifdef __WIN32__
  long winsockinit;
  WSADATA winsock;
  SOCKET fd;
  winsockinit = WSAStartup(0x0101,&winsock);
#else
  int fd;
#endif
  
  //null because we dont use proxy for now
  dest = useproxy(NULL, dest);
  
  //we get the hostname of freedb
  if((h=gethostbyname(dest.hostname))==NULL)
    {
      err = MP3SPLT_FREEDB_ERROR_CANNOT_GET_HOST;
    }
  else
    {
      //we prepare socket
      memset(&host, 0x0, sizeof(host));
      host.sin_family=AF_INET;
      host.sin_addr.s_addr=((struct in_addr *) (h->h_addr)) ->s_addr;
      host.sin_port=htons(dest.port);
      
      //initialize socket
      if((fd=socket(AF_INET, SOCK_STREAM, 0))==-1)
	{
	  err = MP3SPLT_FREEDB_ERROR_INITIALISE_SOCKET;
	}
      else
	{
	  //make connection
	  if ((connect(fd, (void *)&host, sizeof(host))) < 0)
	    {
	      err = MP3SPLT_FREEDB_ERROR_CANNOT_CONNECT;
	    }
	  else
	    {
	      //prepare message to send
	      //proxy not supported for now
	      //if (dest.proxy) {
	      //		sprintf(message,
	      //		"GET http://www.freedb.org"MP3SPLT_SEARCH" "PROXYDLG, search);
	      //		if (dest.auth!=NULL)
	      //		sprintf (message, "%s"AUTH"%s\n", message, dest.auth);
	      //		strncat(message, "\n", 1);
	      //		}
	      //		else 
	      int malloc_number = strlen(MP3SPLT_SEARCH)+strlen(search)+5;
	      if((message = malloc(malloc_number)) == NULL)
		{
		  err = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
		  return err;
		}
	      
	      snprintf(message,malloc_number,
		       "GET "MP3SPLT_SEARCH"\n", search);
		  
	      //message sent
	      if((send(fd, message, strlen(message), 0))==-1) {
		err = MP3SPLT_FREEDB_ERROR_CANNOT_SEND_MESSAGE;
	      }
	      else
		{
		  memset(buffer, 0x00, MP3SPLT_BUFFERSIZE);
		  
		  //we free previous search
		  free_search_results(state->fdb.cdstate,
				      state->fdb.search_results);
		      
		  //create cdstate
		  if ((state->fdb.cdstate = 
		       malloc(sizeof(cd_state))) == NULL)
		    {
		      err = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
		      return err;
		    }
		  state->fdb.cdstate->foundcd = 0;
		      
		  //initialise the search results
		  if ((state->fdb.search_results =
		       malloc (sizeof(freedb_results))) == NULL)
		    {
		      err = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
		      return err;
		    }
		  state->fdb.search_results->number = 0;
		      
		  //we read what we receive from the server
		  do {
		    tot=0;
		    c = buffer;
		    
		    do {
		      i = recv(fd, c, MP3SPLT_BUFFERSIZE-(c-buffer)-1, 0);
		      if (i == -1) 
			{
			  err = MP3SPLT_FREEDB_ERROR_CANNOT_RECV_MESSAGE;
			  return err;
			}
		      tot += i;
		      buffer[tot]='\0';
		      c += i;
		    } while ((i>0)&&(tot<MP3SPLT_BUFFERSIZE-1)
			     &&((e=strstr(buffer, "</html>"))==NULL));
		    
		    //we analyse the buffer
		    tot = analyse_cd_buffer(buffer, tot,
					    state,&err);
		    if (err <= 0)
		      {
			return err;
		      }
		    
		    if (tot == -1) continue;
		    if (tot == -2) break;
		    
		  } while ((i>0)&&(e==NULL)&&(state->fdb.cdstate->foundcd<MP3SPLT_MAXCD));
		  
		  //no cd found
		  if (state->fdb.cdstate->foundcd==0) 
		    err = MP3SPLT_FREEDB_NO_CD_FOUND;
		  //error occured while getting freedb infos
		  if (state->fdb.cdstate->foundcd==-1) 
		    err = MP3SPLT_FREEDB_ERROR_GETTING_INFOS;
		  //max cd number reached
		  if (state->fdb.cdstate->foundcd==MP3SPLT_MAXCD) 
		    err = MP3SPLT_FREEDB_MAX_CD_REACHED;
		}
	      //free memory
	      free(message);
	      closesocket(fd);
	    }
	}
    }
  
  return err;
}

//must only be called after process_freedb_search
//returns the cddb file content corresponding to the last search, for
//the disc_id i (parameter of the function)
//we return possible error in err
//result must be freed
char *get_freedb_file(splt_state *state,
		      int i, int *error)
{
  //possible error that we will return
  *error = MP3SPLT_FREEDB_FILE_OK;
  //the freedb file that we will return
  char *output = NULL;
  
  //socket and internet structures
  struct sockaddr_in host;
  struct hostent *h;
  struct addr dest;
  char *message;
  int tot=0;
  //the buffer that we are using to read incoming transmission
  char buffer[MP3SPLT_BUFFERSIZE];
  //e is used for the end of the buffer
  //c is used for the buffer read
  char *c, *e=NULL;
  
  //fd = socket identifier
#ifdef __WIN32__
  long winsockinit;
  WSADATA winsock;
  SOCKET fd;
  winsockinit = WSAStartup(0x0101,&winsock);
#else
  int fd;
#endif
  
  //null because we dont use proxy for now
  dest = useproxy(NULL, dest);
  
  //we get the hostname of freedb
  if((h=gethostbyname(dest.hostname))==NULL)
    {
      *error = MP3SPLT_FREEDB_ERROR_CANNOT_GET_HOST;
    }
  else
    {
      //we prepare socket
      memset(&host, 0x0, sizeof(host));
      host.sin_family=AF_INET;
      host.sin_addr.s_addr=((struct in_addr *) (h->h_addr)) ->s_addr;
      host.sin_port=htons(dest.port);
      
      //prepare message to send
      //proxy not yet supported
      //      if (dest.proxy) {
      //	sprintf(message, "GET "FREEDBHTTP"cmd=cddb+read+%s+%s&hello=nouser+mp3splt.net+"PACKAGE_NAME"+"PACKAGE_VERSION"&proto=5 "PROXYDLG, 
      //	cdstate->discs[i].category, cdstate->discs[i].discid);
      //	if (dest.auth!=NULL) {
      //	sprintf (message, "%s"AUTH"%s\n", message, dest.auth);
      //	memset(dest.auth, 0x00, strlen(dest.auth));
      //	free(dest.auth);
      //	}
      //	strncat(message, "\n", 1);
      //	}
      //	else {
      int malloc_number = strlen(state->fdb.cdstate->discs[i].category) +
	strlen(state->fdb.cdstate->discs[i].discid) + 20;
      message = malloc(malloc_number);
      if (message != NULL)
	{
	  snprintf(message, malloc_number,
		   "CDDB READ %s %s\n", 
		   state->fdb.cdstate->discs[i].category, 
		   state->fdb.cdstate->discs[i].discid);
	  host.sin_port=htons(MP3SPLT_PORT2);
	  //}
	  
	  //open socket
	  if((fd=socket(AF_INET, SOCK_STREAM, 0))==-1)
	    {
	      *error = MP3SPLT_FREEDB_ERROR_CANNOT_OPEN_SOCKET;
	    }
	  else
	    {
	      //connect to host
	      if ((connect(fd, (void *)&host, sizeof(host)))==-1)
		{
		  *error = MP3SPLT_FREEDB_ERROR_CANNOT_CONNECT;
		}
	      else
		{
		  //possible errors + proxy
		  if (!dest.proxy) {
		    i=recv(fd, buffer, MP3SPLT_BUFFERSIZE-1, 0);
		    if (i == -1)
		      {
			*error = MP3SPLT_FREEDB_ERROR_CANNOT_RECV_MESSAGE;
		      }
		    buffer[i]='\0';
		    
		    if (strncmp(buffer,"201",3)!=0)  
		      {
			*error = MP3SPLT_FREEDB_ERROR_SITE_201;
		      }
		    
		    if((send(fd, MP3SPLT_HELLO, strlen(MP3SPLT_HELLO), 0))==-1)
		      {
			*error = MP3SPLT_FREEDB_ERROR_CANNOT_SEND_MESSAGE;
		      }
		    i=recv(fd, buffer, MP3SPLT_BUFFERSIZE-1, 0);
		    if (i == -1)
		      {
			*error = MP3SPLT_FREEDB_ERROR_CANNOT_RECV_MESSAGE;
		      }
		    buffer[i]='\0';
    
		    if (strncmp(buffer,"200",3)!=0)  
		      *error = MP3SPLT_FREEDB_ERROR_SITE_200;
		  }
	      
		  //we send the message
		  if((send(fd, message, strlen(message), 0))==-1)
		    {
		      *error = MP3SPLT_FREEDB_ERROR_CANNOT_SEND_MESSAGE;
		    }
		  else
		    {
		      memset(buffer, 0x00, MP3SPLT_BUFFERSIZE);
		      c = buffer;
		      tot=0;
		      
		      //we read 
		      do {
			i = recv(fd, c, MP3SPLT_BUFFERSIZE-(c-buffer)-1, 0);
			if (i == -1)
			  {
			    break;
			  }
			tot += i;
			buffer[tot]='\0';
			c += i;
		      } while ((i>0)&&(tot<MP3SPLT_BUFFERSIZE-1)&&((e=strstr(buffer, "\n."))==NULL));
		      
		      //we quit
		      if (!dest.proxy)
			if((send(fd, "quit\n", 5, 0))==-1)
			  {
			    *error = MP3SPLT_FREEDB_ERROR_CANNOT_SEND_MESSAGE;
			  }
		    }
		}
	      closesocket(fd);
	    }
	  
	  if (tot==0) 
	    {
	      *error = MP3SPLT_FREEDB_ERROR_BAD_COMMUNICATION;
	    }
	  
	  if (e!=NULL)
	    {
	      buffer[e-buffer+1]='\0';
	    }
  
	  //if invalid server answer
	  if ((strstr(buffer, "database entry follows"))==NULL)
	    {
	      if ((c = strchr (buffer, '\n'))!=NULL)
		{
		  buffer[c-buffer]='\0';
		}
	      *error = MP3SPLT_FREEDB_ERROR_INVALID_SERVER_ANSWER;
	    }
	  else
	    {
	      if ((c = strchr (buffer, '#'))==NULL)
		{
		  return NULL;
		}
	      
	      output = malloc(strlen(buffer));
	      if (output != NULL)
		{
		  sprintf (output,c);
		}
	      else
		{
		  *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
		}
	    }
	  
	  //we free memory
	  free(message);
	}
      else
	{
	  *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
	}
    }

  return output;
}

//deprecated, and not in use
//but may useful for the implementation of the proxy
/*int search_freedb (splt_state *state)
{
	char *c, *e=NULL;
	FILE *output = NULL;
	struct sockaddr_in host;
	struct hostent *h;
	struct addr dest;

	if ((c=getenv("HOME"))!=NULL) sprintf(message, "%s/"PROXYCONFIG, c);
	else strncpy(message, PROXYCONFIG, strlen(PROXYCONFIG));

	if (!(output=fopen(message, "r"))) {
		if (!(output=fopen(message, "w+"))) {
			fprintf(stderr, "\nWARNING Can't open config file ");
			perror(message);
		}
		else {
			fprintf (stderr, "Will you use a proxy? (y/n): ");
			fgets(junk, 200, stdin);
			if (junk[0]=='y') {
				fprintf (stderr, "Proxy Address: ");
				fgets(junk, 200, stdin);
				fprintf (output, "PROXYADDR=%s", junk);
				fprintf (stderr, "Proxy Port: ");
				fgets(junk, 200, stdin);
				fprintf (output, "PROXYPORT=%s", junk);
				fprintf (stderr, "Need authentication? (y/n): ");
				fgets(junk, 200, stdin);
				if (junk[0]=='y') {
					fprintf (output, "PROXYAUTH=1\n");
					fprintf (stderr, "Would you like to save password (insecure)? (y/n): ");
					fgets(junk, 200, stdin);
					if (junk[0]=='y') {
						login (message);
						e = b64(message, strlen(message));
						fprintf (output, "%s\n", e);
						memset(message, 0x00, strlen(message));
						memset(e, 0x00, strlen(e));
						free(e);
					}
				}
			}
		}
	}
	
	if (state->fdb.cdstate->foundcd<=0) {
		if (dest.proxy) {
			if (strstr(buffer, "HTTP/1.0")!=NULL) {
				if ((c = strchr (buffer, '\n'))!=NULL)
					buffer[c-buffer]='\0';
				fprintf (stderr, "Proxy Reply: %s\n", buffer);
			}
		}
	}
	return 0;
}*/

