/**********************************************************
 *
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 * http://mp3splt.sourceforge.net
 *
 *********************************************************/

/**********************************************************
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *
 *********************************************************/

#include <sys/stat.h>
#include <string.h>

#include "mp3splt_utils.h"

//if the new_filename_path is "", we put the directory of
//the current song
//result must be freed
//returns NULL means he cannot allocate memory
char *check_put_dir_of_cur_song(char *filename,
				char *the_filename_path)
{
  char *c = NULL;
  char *filename_path = NULL;
  int length_malloc = strlen(filename)+1;
  if (length_malloc < 5)
    {
      length_malloc = 8;
    }
  
  if((filename_path = malloc(length_malloc)) == NULL)
    {
      return NULL;
    }
  
  int cur_dir = MP3SPLT_FALSE;
  //if new filename path is null
  if (the_filename_path == NULL)
    {
      cur_dir = MP3SPLT_TRUE;
    }
  else
    {
      //if new_filename_path is ""
      if (strcmp(the_filename_path,"") == 0)
	{
	  cur_dir = MP3SPLT_TRUE;
	}
    }
  
  //if we split in the current directory
  if (cur_dir)
    {
      snprintf(filename_path,length_malloc,
	       "%s", filename);
      c = strrchr(filename,MP3SPLT_DIRCHAR);
      //if we found a dirchar, erase everythig after to get the path
      if (c != NULL)
	{
	  *c = '\0';
	}
      //otherwise we set the path as the current directory
      else
	{
	  snprintf(filename_path, length_malloc,
		   ".%c", MP3SPLT_DIRCHAR);
	}
      return filename_path;
    }
  
  free(filename_path);
  
  return strdup(the_filename_path);
}

//checks if the splitpoints are inferior to the length of the song
//remove all the splitpoint superior than the biggest one superior
//to the end of file
void check_splitpts_inf_song_length(splt_state *state,
				    int *error)
{
  int i = 0;
  int found_max_splitpoint = MP3SPLT_FALSE;
  int initial_splitpoints = state->split.splitnumber;
  
  //check if splitpoints > total time
  for(i = 0; i < initial_splitpoints; i++)
    {
      if (state->split.splitpoints[i] > 
	  state->split.total_time)
	{
	  if (found_max_splitpoint)
	    {
	      state->split.splitpoints[i] = 0;
	      state->split.splitnumber--;
	    }
	  else
	    {
	      found_max_splitpoint = MP3SPLT_TRUE;
	      *error = MP3SPLT_SPLITPOINT_BIGGER_THAN_LENGTH;
	    }
	}
    }
}

//checks if splitpoints are in order : 1, 5, 6 ...
//and not 1, 5, 4 ... (4<5 -> error)
void check_if_splitpoints_in_order(splt_state *state,
				   int *error)
{
  int i;

  for(i = 0; i < (state->split.splitnumber -1); i++)
    {
      //check if first splitpoint is positive
      if (state->split.splitpoints[i] < 0)
	{
	  *error = MP3SPLT_ERROR_NEGATIVE_SPLITPOINT;
	  return;
	}
      
      //check if splitpoints not in order
      if (state->split.splitpoints[i] > state->split.splitpoints[i+1])
	{
	  *error = MP3SPLT_ERROR_SPLITPOINTS_NOT_IN_ORDER;
	  return;
	}
      else
	{
	  //check if two consecutive splitpoints are equals
	  if (state->split.splitpoints[i] == state->split.splitpoints[i+1])
	    {
	      *error = MP3SPLT_ERROR_EQUAL_SPLITPOINTS;
	      return;
	    }
	}
    }
}

//checks if the filename path is correct
void check_if_new_filename_path_correct(char *new_filename_path,
					int *error)
{
  //if the path equals "", then dont do the directory check,
  //we will split in the current directory
  if (strcmp(new_filename_path, "") != 0)
    {
      //used to see if the file exists
      struct stat buffer;
      int         status;
      
      //-1 means error
      if((status = stat(new_filename_path, &buffer))
	 == -1)
	{
	  *error = MP3SPLT_ERROR_INCORRECT_PATH;
	  return;
	}
      
      //if it is a directory
      if (S_ISDIR(buffer.st_mode) != 0)
	{
	  //no error
	  return;
	}
      else
	{
	  *error = MP3SPLT_ERROR_INCORRECT_PATH;
	}
    }
}

//checks if the options are compatible. The implementation of this
//function is not essential
int check_compatible_options(splt_state *state)
{
  //TODO
  
  return MP3SPLT_TRUE;
}

//checks if we have incorrect options and sets the correct options if
//we need so (this function is quite important)
void check_set_correct_options(splt_state *state)
{
  //if we have the silence option or the adjustoption
  //we set the default values if something is not ok
  if ((state->options.split_mode == 
       MP3SPLT_OPTION_SILENCE_MODE)
      || state->options.option_auto_adjust)
    {
      //if we have the adjust option or the silence option,
      //enable frame mode by default
      state->options.option_mp3_frame_mode = MP3SPLT_TRUE;
      
      //for the autoadjust or silence mode
      if  ((state->options.parameter_threshold<-96.f) || 
	   (state->options.parameter_threshold>0.f))
	{
	  state->options.parameter_threshold = 
	    MP3SPLT_DEFAULT_PARAM_THRESHOLD;
	}
      if  ((state->options.parameter_offset<-2.f) || 
	   (state->options.parameter_offset>2.f))
	{
	  state->options.parameter_offset = 
	    MP3SPLT_DEFAULT_PARAM_OFFSET;
	}
      //for the adjust mode
      if (state->options.parameter_gap < 0)
	{
	  state->options.parameter_gap = 
	    MP3SPLT_DEFAULT_PARAM_GAP;
	}
      //for the silence mode
      if (state->options.parameter_minimum_length<0.f)
	{
	  state->options.parameter_minimum_length = 
	    MP3SPLT_DEFAULT_PARAM_MINIMUM_LENGTH;
	  //disable auto adjust mode
	  state->options.option_auto_adjust = MP3SPLT_FALSE;
	}
    }
  
  //if we don't have adjust, set gap to 0
  if (!state->options.option_auto_adjust)
    {
      state->options.parameter_gap = 0;
    }
  
  //if seekable and (silence or adjust or wrap or err sync)
  if ((state->options.option_input_not_seekable) &&
      (state->options.option_auto_adjust ||
       (state->options.split_mode == MP3SPLT_OPTION_SILENCE_MODE) ||
       (state->options.split_mode == MP3SPLT_OPTION_MP3_ERROR_MODE) ||
       (state->options.split_mode == MP3SPLT_OPTION_WRAP_MODE)))
    {
      state->options.option_input_not_seekable = MP3SPLT_FALSE;
    }
}

//checks if mp3 file or ogg file
//returns possible error
void check_if_mp3_or_ogg(splt_state *state,
			 char *filename,
			 int *error)
{
#ifndef NO_OGG
  //the file to open
  FILE *file_input;
  OggVorbis_File ogg_file;
  
  if(!(file_input = fopen(filename, "rb")))
    {
      *error = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
      return;
    }
  //if we can open the file, check if ogg or mp3
  else 
    {
      //supposing that if not ogg then mp3 (mp3 <=> ogg = 0)
      if ((ov_test(file_input, &ogg_file, NULL, 0) + 1) != 1)
	{
	  state->file_format = MP3SPLT_MP3_FORMAT;
	}
      else
	{
	  state->file_format = MP3SPLT_OGG_FORMAT;
	}
    
      //if the file was opened, we close it
      //properly with ov_clear
      if(state->file_format == MP3SPLT_OGG_FORMAT)
	{
	  ov_clear(&ogg_file);
	}
    }
  
  //if file is not ogg, closing the file normally
  if (state->file_format != MP3SPLT_OGG_FORMAT)
    {
      fclose(file_input);
    }
#else
  //we have to check if mp3 file !!!
  state->file_format = MP3SPLT_MP3_FORMAT;
#endif
}
