/**********************************************************
 *
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 * http://mp3splt.sourceforge.net
 *
 *********************************************************/

/**********************************************************
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *
 *********************************************************/

//see types.h for error codes and structures
#include "mp3splt_types.h"

//we include config.h
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

//use PACKAGE_NAME variable for the name of the program
//use PACKAGE_VERSION variable for the version of the program
#ifndef MP3SPLT_PACKAGE_NAME
#define MP3SPLT_PACKAGE_NAME "libmp3splt"
#endif
#ifndef MP3SPLT_PACKAGE_VERSION
#define MP3SPLT_PACKAGE_VERSION "0.3.1"
#endif
#define MP3SPLT_AUTHOR "Matteo Trotta | Munteanu Alexandru"
#define MP3SPLT_EMAIL "<matteo.trotta@lib.unimib.it> <io_alex_2002@yahoo.fr>"
#define MP3SPLT_WEBSITE "http://mp3splt.sourceforge.net"

/************************************/
/* Initialisation and free          */

//creates a new state structure, needed by libmp3splt
splt_state *libmp3splt_create_new_state(int *error);

//this function frees the left variables in the library
//call this function ONLY at the end of the program
//and don't forget to call it
void libmp3splt_free_left_variables(splt_state *state);

/************************************/
/* Check functions                  */

//returns if the library has id3tag support or not
int libmp3splt_has_libid3tag();
//returns if the library has ogg support or not
int libmp3splt_has_ogg();

/************************************/
/* Split functions                  */

//puts a splitpoint
void libmp3splt_put_splitpoint(splt_state *state, int number,
			       long splitpoint, char *name);

//split a ogg or mp3 file
int libmp3splt_split_file(splt_state *state, char *filename, 
			  int splitnumber, char *the_filename_path);
//cancel split function
void libmp3splt_stop_split();

//returns the number of syncerrors found
//for the file filename and returns possible errors 
//as sync_result variable
int libmp3splt_get_mp3_syncerrors_number(splt_state *state,
					 char *filename,
					 int *sync_result);
//returns the wrapped files found
char **libmp3splt_get_wrap_files(splt_state *state,
				 char *filename,
				 int *process_result,
				 int *wrap_files_number);

/************************************/
/*    Cddb and Cue functions        */

//get the cue splitpoints from a file
int libmp3splt_get_cue_splitpoints_from_file(splt_state *state,
					     char *file,
					     int *err);
//read cddb splitpoints from file and put them in the
//state->splitpoints
int libmp3splt_get_cddb_splitpoints_from_file(splt_state *state,
					      char *file,
					      int *err);

/************************************/
/*    Freedb functions              */

//returns the freedb results and possible eerror
freedb_results *libmp3splt_get_freedb_search(splt_state *state,
					     //our search
					     char *search,
					     //possible errors
					     int *error);
void libmp3splt_write_freedb_file_result(splt_state *state, int disc_id,
					 char *cddb_file, int *err);

//string s is freed, call with strdup for example
void libmp3splt_checkparse_outformat(splt_state *state,
				     char *s,
				     int *error);

//simple example on using the library until further documentation
//example simple program using the libmp3splt version 0.3
/*
//if this file is called test.c, 
//compile like this for example : 
//gcc -g -Wall -o test test.c -lmp3splt
#include <libmp3splt/mp3splt.h>

int main(void)
{
  int error;
  splt_state *a;
  
  //we initialise our new state
  a = libmp3splt_create_new_state (&error);
  
  //two splitpoints, splits the first 100 seconds from
  //the beginning of the song (data is in hundreths of secs)
  a->splitpoints[0] = 0000.0;
  a->splitpoints[1] = 10000.0;
  
  //name of the output file (must be without extension)
  //(file for the splitpoints 0->1)
  snprintf(a->fn[0],255,"output");
  
  //split the file "input.mp3" from the current directory
  //in the current directory
  //2 is the number of splitpoints
  error = libmp3splt_split_file ("input.mp3", 
				 a, 2, "");
  
  //free variables in the library
  libmp3splt_free_left_variables (a);
  
  return 0;
}
*/
