/**********************************************************
 *
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 * http://mp3splt.sourceforge.net
 *
 *********************************************************/

/**********************************************************
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *
 *********************************************************/

#include <time.h>
#include <string.h>
#include <math.h>
#include <locale.h>
#include <values.h>

#ifndef NO_OGG
 
#include "mp3splt_utils.h"

#if !HAVE_FSEEKO
#define fseeko fseek
#define ftello ftell
#endif

//ogg genre list
const char *genre_list[] = {
"Blues",                 "Classic Rock",          "Country",
"Dance",                 "Disco",                 "Funk",
"Grunge",                "Hip-Hop",               "Jazz",
"Metal",                 "New Age",               "Oldies",
"Other",                 "Pop",                   "R&B",
"Rap",                   "Reggae",                "Rock",
"Techno",                "Industrial",            "Alternative",
"Ska",                   "Death Metal",           "Pranks",
"Soundtrack",            "Euro-Techno",           "Ambient",
"Trip-Hop",              "Vocal",                 "Jazz+Funk",
"Fusion",                "Trance",                "Classical",
"Instrumental",          "Acid",                  "House",
"Game",                  "Sound Clip",            "Gospel",
"Noise",                 "Alt. Rock",             "Bass",
"Soul",                  "Punk",                  "Space",
"Meditative",            "Instrum. Pop",          "Instrum. Rock",
"Ethnic",                "Gothic",                "Darkwave",
"Techno-Indust.",        "Electronic",            "Pop-Folk",
"Eurodance",             "Dream",                 "Southern Rock",
"Comedy",                "Cult",                  "Gangsta",
"Top 40",                "Christian Rap",         "Pop/Funk",
"Jungle",                "Native American",       "Cabaret",
"New Wave",              "Psychadelic",           "Rave",
"Showtunes",             "Trailer",               "Lo-Fi",
"Tribal",                "Acid Punk",             "Acid Jazz",
"Polka",                 "Retro",                 "Musical",
"Rock & Roll",           "Hard Rock",             "Folk",
"Folk/Rock",             "National Folk",         "Swing",
"Fusion",                "Bebob",                 "Latin",
"Revival",               "Celtic",                "Bluegrass",
"Avantgarde",            "Gothic Rock",           "Progress. Rock",
"Psychadel. Rock",       "Symphonic Rock",        "Slow Rock",
"Big Band",              "Chorus",                "Easy Listening",
"Acoustic",              "Humour",                "Speech",
"Chanson",               "Opera",                 "Chamber Music",
"Sonata",                "Symphony",              "Booty Bass",
"Primus",                "Porn Groove",           "Satire",
"Slow Jam",              "Club",                  "Tango",
"Samba",                 "Folklore",              "Ballad",
"Power Ballad",          "Rhythmic Soul",         "Freestyle",
"Duet",                  "Punk Rock",             "Drum Solo",
"A Capella",             "Euro-House",            "Dance Hall",
"Goa",                   "Drum & Bass",           "Club-House",
"Hardcore",              "Terror",                "Indie",
"BritPop",               "Negerpunk",             "Polsk Punk",
"Beat",                  "Christian Gangsta Rap", "Heavy Metal",
"Black Metal",           "Crossover",             "Contemporary Christian",
"Christian Rock",        "Merengue",              "Salsa",
"Trash Metal",           "Anime",                 "Jpop",
"Synthpop"};

//put the original ogg tags
void put_original_ogg_tags(splt_state *state,
			   int *tag_error)
{
  vorbis_comment *vc_local;
  vc_local = ov_comment(&state->ostate->vf,-1);
  char *a,*t,*al,*da,*g,*tr,*com;
  a = vorbis_comment_query(vc_local, "artist",0);
  t = vorbis_comment_query(vc_local, "title",0);
  al = vorbis_comment_query(vc_local, "album",0);
  da = vorbis_comment_query(vc_local, "date",0);
  g = vorbis_comment_query(vc_local, "genre",0);
  tr = vorbis_comment_query(vc_local, "tracknumber",0);
  com = vorbis_comment_query(vc_local, "comment",0);
  vorbis_comment_clear(&state->ostate->vc);
  vorbis_comment_init(&state->ostate->vc);
  v_comment(&state->ostate->vc,
	    a,al,t,tr,da,g,com);
}

//puts the ogg tags
void put_ogg_tags(splt_state *state,
		  int *error)
{
  //clean_original_id3(state);
  //if we put the original tags
  vorbis_comment_clear(&state->ostate->vc);
  if (state->options.tags == MP3SPLT_TAGS_ORIGINAL_FILE)
    {
      int tag_error = 0;
      put_original_ogg_tags(state,&tag_error);
      if (tag_error < 0)
	{
	  *error = tag_error;
	}
    }
  else
    {
      //if we put current tags (cddb,cue,...)
      if (state->options.tags == MP3SPLT_CURRENT_TAGS)
	{
	  //only if we have the artist or the title
	  if ((state->tags.id[state->split.current_split].title[0] != '\0')
	      ||(state->tags.id[state->split.current_split].artist[0] != '\0'))
	    {
	      v_comment(&state->ostate->vc,
			state->tags.id[state->split.current_split].artist,
			state->tags.id[state->split.current_split].album,
			state->tags.id[state->split.current_split].title,
			trackstring(state->split.current_split+1),
			state->tags.id[state->split.current_split].year,
			(char *)genre_list[(int) 
					   state->tags.id[state->split.current_split].genre],
			state->tags.id[state->split.current_split].comment);
	    }
	}
      //no tags
    }
}

//saves a packet
static v_packet *save_packet(ogg_packet *packet)
{
  v_packet *p = malloc(sizeof(v_packet));
  
  p->length = packet->bytes;
  p->packet = malloc(p->length);
  memcpy(p->packet, packet->packet, p->length);
  
  return p;
}

//frees a packet
static void free_packet(v_packet *p)
{
  if(p)
    {
      if(p->packet)
	free(p->packet);
      free(p);
    }
}

static long get_blocksize(ogg_state *s, vorbis_info *vi, ogg_packet *op)
{
  int this = vorbis_packet_blocksize(vi, op);
  int ret = (this+s->prevW)/4;
  
  s->prevW = this;
  return ret;
}

static int update_sync(ogg_sync_state *sync_in, FILE *f)
{
  char *buffer = ogg_sync_buffer(sync_in, MP3SPLT_BUFSIZE);
  int bytes = fread(buffer,1,MP3SPLT_BUFSIZE,f);
  ogg_sync_wrote(sync_in, bytes);
  return bytes;
}

/* Returns 0 for success, or -1 on failure. */
static int write_pages_to_file(ogg_stream_state *stream, FILE *file, int flush)
{
  ogg_page page;

  if(flush)
    {
      while(ogg_stream_flush(stream, &page))
	{
	  if(fwrite(page.header,1,page.header_len, file) != page.header_len)
	    return -1;
	  if(fwrite(page.body,1,page.body_len, file) != page.body_len)
	    return -1;
	}
    }
  else
    {
      while(ogg_stream_pageout(stream, &page))
	{
	  if(fwrite(page.header,1,page.header_len, file) != page.header_len)
	    return -1;
	  if(fwrite(page.body,1,page.body_len, file) != page.body_len)
	    return -1;
	}
    }

  return 0;
}


/* Read stream until we get to the appropriate cut point.
 *
 * We need to do the following:
 *   - Save the final two packets in the stream to temporary buffers.
 *     These two packets then become the first two packets in the 2nd stream
 *     (we need two packets because of the overlap-add nature of vorbis).
 *   - For each packet, buffer it (it could be the 2nd last packet, we don't
 *     know yet (but we could optimise this decision based on known maximum
 *     block sizes, and call get_blocksize(), because this updates internal
 *     state needed for sample-accurate block size calculations.
 */
static int find_begin_cutpoint(ogg_state *s, FILE *in, ogg_int64_t cutpoint)
{
  int eos=0;
  ogg_page page;
  ogg_packet packet;
  ogg_int64_t granpos, prevgranpos;
  int result;

  granpos = prevgranpos = 0;
  
  while(!eos)
    {
      while(!eos)
	{
	  int result = ogg_sync_pageout(s->sync_in, &page);
	  if(result==0) break;
	  else 
	    if(result>0)
	      {
		granpos = ogg_page_granulepos(&page);
		ogg_stream_pagein(s->stream_in, &page);
		
		if(granpos < cutpoint)
		  {
		    while(1)
		      {
			result=ogg_stream_packetout(s->stream_in, &packet);
			/* throw away result, but update state */
			get_blocksize(s,s->vi,&packet);

			if(result==0) 
			  {
			    break;
			  }
			else 
			  {
			    if(result!=-1)
			      {
				/* We need to save the last packet in the first
				 * stream - but we don't know when we're going
				 * to get there. So we have to keep every packet
				 * just in case.
				 */
				if(s->packets[0])
				  free_packet(s->packets[0]);
				s->packets[0] = save_packet(&packet);
			      }
			  }
		      }
		    prevgranpos = granpos;
		  }
		else
		  {
		    eos=1; /* First stream ends somewhere in this page.
			      We break of out this loop here. */
		  }
		
		if(ogg_page_eos(&page))
		  {
		    eos=1;
		  }
	      }
	}
      if(!eos)
	{
	  if(update_sync(s->sync_in, in)==0)
	    {
	      eos=1;
	    }
	}
    }
  
  /* Now, check to see if we reached a real EOS */
  if(granpos < cutpoint)
    {
      return -1; // Cutpoint is out of file
    }
  
  while((result = ogg_stream_packetout(s->stream_in, &packet))!=0)
    {
      int bs;
      
      bs = get_blocksize(s, s->vi, &packet);
      prevgranpos += bs;

      if(prevgranpos > cutpoint)
	{
	  s->packets[1] = save_packet(&packet);
	  break;
	}
      if(s->packets[0])
	free_packet(s->packets[0]);
      s->packets[0] = save_packet(&packet);
    }
  
  /* Remaining samples in first packet */
  s->initialgranpos = prevgranpos - cutpoint;
  s->cutpoint_begin = cutpoint;
  
  return 0;
}

/* Process second stream.
 *
 * We need to do more packet manipulation here, because we need to calculate
 * a new granulepos for every packet, since the old ones are now invalid.
 * Start by placing the modified first and second packets into the stream.
 * Then just proceed through the stream modifying packno and granulepos for
 * each packet, using the granulepos which we track block-by-block.
 */
static int find_end_cutpoint(splt_state *sp_state, ogg_stream_state *stream, FILE *in, FILE *f, 
			     ogg_int64_t cutpoint, short adjust, float threshold)
{
  put_progress_text(sp_state,MP3SPLT_PROGRESS_CREATE);
  
  //for the progress
  int progress_adjust = 1;
  
  ogg_state *s = sp_state->ostate;
  
  ogg_packet packet;
  ogg_page page;
  int eos=0;
  int result;
  ogg_int64_t page_granpos = 0, current_granpos = 0, prev_granpos = 0;
  ogg_int64_t packetnum=0; /* Should this start from 0 or 3 ? */
  
  if(s->packets[0] && s->packets[1])
    { // Check if we have the 2 packet, begin can be 0!
      packet.bytes = s->packets[0]->length;
      packet.packet = s->packets[0]->packet;
      packet.b_o_s = 0;
      packet.e_o_s = 0;
      packet.granulepos = 0;
      packet.packetno = packetnum++;
      ogg_stream_packetin(stream,&packet);

      packet.bytes = s->packets[1]->length;
      packet.packet = s->packets[1]->packet;
      packet.b_o_s = 0;
      packet.e_o_s = 0;
      packet.granulepos = s->initialgranpos;
      packet.packetno = packetnum++;
      ogg_stream_packetin(stream,&packet);

      if(ogg_stream_flush(stream, &page)!=0)
	{
	  fwrite(page.header,1,page.header_len,f);
	  fwrite(page.body,1,page.body_len,f);
	}

      while(ogg_stream_flush(stream, &page)!=0)
	{
	  /* Might this happen for _really_ high bitrate modes, if we're
	   * spectacularly unlucky? Doubt it, but let's check for it just
	   * in case.
	   */
	  //fprintf(stderr, "Warning: First audio packet didn't fit into page. File may not decode correctly\n");
	  fwrite(page.header,1,page.header_len,f);
	  fwrite(page.body,1,page.body_len,f);
	}
    }
  else s->initialgranpos = 0;

  current_granpos = s->initialgranpos;
  
  while(!eos)
    {
      while(!eos)
	{
	  result=ogg_sync_pageout(s->sync_in, &page);
	  if(result==0) break;
	  else
	    if(result!=-1)
	      {
		page_granpos = ogg_page_granulepos(&page) - s->cutpoint_begin;
		if(ogg_page_eos(&page)) eos=1;
		ogg_stream_pagein(s->stream_in, &page);
		
		if ((cutpoint == 0) || (page_granpos < cutpoint))
		  {
		    while(1)
		      {
			result = ogg_stream_packetout(s->stream_in, &packet);
			if(result==0) break;
			else
			  if(result!=-1)
			    {
			      int bs = get_blocksize(s, s->vi, &packet);
			      current_granpos += bs;
			      if(s->packets[0]) /* We need to save packet to optimize following split process */
				free_packet(s->packets[0]);
			      s->packets[0] = save_packet(&packet);
			      if(current_granpos > page_granpos)
				current_granpos = page_granpos;
			      packet.granulepos = current_granpos;
			      packet.packetno = packetnum++;
			      
			      //progress
			      if (progress_adjust)
				{
				  if ((sp_state->options.split_mode
				       == MP3SPLT_OPTION_SILENCE_MODE) ||
				      (!sp_state->options.option_auto_adjust))
				    {
				      update_progress(sp_state, (float)page_granpos,
						      (float)cutpoint,
						      1,0,MP3SPLT_DEFAULT_PROGRESS_RATE);
				    }
				  else
				    {
				      update_progress(sp_state, (float)page_granpos,
						      (float)cutpoint,
						      2,0,MP3SPLT_DEFAULT_PROGRESS_RATE);
				    }
				}
			      
			      ogg_stream_packetin(stream, &packet);
			      if(write_pages_to_file(stream,f, 0))
				return -1;
			      
			    }
		      }
		    prev_granpos = page_granpos;
		  }
		else {
		  if (adjust) {
		    if (ogg_scan_silence(sp_state,
					 (2 * adjust), threshold, 0.f, 0, &page, current_granpos) > 0)
		      {
			cutpoint = (silence_position(s->silence_list, s->off) * s->vi->rate);
		      }
		    else
		      {
			cutpoint = (cutpoint + (adjust * s->vi->rate));
		      }
		    ssplit_free(&s->silence_list);
		    adjust=0;
		    progress_adjust = 0;
		    put_progress_text(sp_state,MP3SPLT_PROGRESS_CREATE);
		  }
		  else 
		    {
		      eos=1; /* We reached the second cutpoint */
		    }
		}
		if(ogg_page_eos(&page))
		  {
		    eos=1;
		  }
	      }
	}
      if(!eos)
	{
	  if(update_sync(s->sync_in, in)==0)
	    {
	      eos=1;
	    }
	}
    }

  if ((cutpoint == 0) || (page_granpos < cutpoint)) // End of file. We stop here
    {
      if(write_pages_to_file(stream,f, 0))
	return -1;
      s->end = -1; // No more data available. Next processes aborted
      return 0;
    }

  while((result = ogg_stream_packetout(s->stream_in, &packet))!=0)
    {
      int bs;
      bs = get_blocksize(s, s->vi, &packet);
      prev_granpos += bs;
      
      if(prev_granpos >= cutpoint)
	{
	  //we free possible previous packets
	  if (s->packets[1])
	    free_packet(s->packets[1]);
	  s->packets[1] = save_packet(&packet);
	  packet.granulepos = cutpoint; /* Set it! This 'truncates' the final packet, as needed. */
	  packet.e_o_s = 1;
	  ogg_stream_packetin(stream, &packet);
	  break;
	}
      
      if(s->packets[0])
	free_packet(s->packets[0]);
      s->packets[0] = save_packet(&packet);

      ogg_stream_packetin(stream, &packet);
      if(write_pages_to_file(stream,f, 0))
	return -1;
    }

  if(write_pages_to_file(stream,f, 0))
    return -1;
  
  s->initialgranpos = prev_granpos - cutpoint;
  s->end = 1;
  s->cutpoint_begin += cutpoint;

  return 0;
}

static void submit_headers_to_stream(ogg_stream_state *stream, ogg_state *s)
{
  int i;
  for(i=0;i<3;i++)
    {
      ogg_packet p;
      p.bytes = s->headers[i]->length;
      p.packet = s->headers[i]->packet;
      p.b_o_s = ((i==0)?1:0);
      p.e_o_s = 0;
      p.granulepos=0;
      
      ogg_stream_packetin(stream, &p);
    }
}

/* Pull out and save the 3 header packets from the input file.
 */

static int process_headers(ogg_state *s)
{
  ogg_page page;
  ogg_packet packet;
  int bytes;
  int i;
  char *buffer;

  ogg_sync_init(s->sync_in);
  
  vorbis_info_init(s->vi);
  vorbis_comment_init(&s->vc);
  
  while (ogg_sync_pageout(s->sync_in, &page)!=1)
    {
      buffer = ogg_sync_buffer(s->sync_in, MP3SPLT_BUFSIZE);
      bytes = fread(buffer, 1, MP3SPLT_BUFSIZE, s->in);
      if (bytes <= 0)
	{
	  return -1;
	}
      ogg_sync_wrote(s->sync_in, bytes);
    }
  
  s->serial = ogg_page_serialno(&page);
  
  ogg_stream_init(s->stream_in, s->serial);

  if(ogg_stream_pagein(s->stream_in, &page) <0)
    {
      return -1;
    }

  if(ogg_stream_packetout(s->stream_in, &packet)!=1){
    return -1;
  }

  if(vorbis_synthesis_headerin(s->vi, &s->vc, &packet)<0)
    {
      return -1;
    }
  
  s->headers[0] = save_packet(&packet);
  
  i=0;
  while(i<2)
    {
      while(i<2) {
	int res = ogg_sync_pageout(s->sync_in, &page);
	if(res==0)break;
	if(res==1)
	  {
	    ogg_stream_pagein(s->stream_in, &page);
	    while(i<2)
	      {
		res = ogg_stream_packetout(s->stream_in, &packet);
		if(res==0)break;
		if(res<0)
		  {
		    return -1;
		  }
		s->headers[i+1] = save_packet(&packet);
		vorbis_synthesis_headerin(s->vi,&s->vc,&packet);
		i++;
	      }
	  }
      }
      buffer=ogg_sync_buffer(s->sync_in, MP3SPLT_BUFSIZE);
      bytes=fread(buffer,1,MP3SPLT_BUFSIZE,s->in);
      if(bytes==0 && i<2)
	{
	  return -1;
	}
      ogg_sync_wrote(s->sync_in, bytes);
    }
  
  return 0;
}

ogg_state *ogginfo(FILE *in, ogg_state *state,int *error)
{
  state = v_new();
  
  if (state == NULL) {
    *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
    return NULL;
  }
  
  state->in = in;
  state->end = 0;
  
  if (state->in != stdin)
    {
      if(ov_open(state->in, &state->vf, NULL, 0) < 0)
	{
	  *error = MP3SPLT_ERROR_INVALID_OGG;
	  return NULL;
	}
      rewind(state->in);
    }
  
  /* Read headers in, and save them */
  if(process_headers(state))
    {
      *error = MP3SPLT_ERROR_INVALID_OGG;
      return NULL;
    }
  
  if (state->in != stdin)
    {
      state->len = (ogg_int64_t) ((state->vi->rate)*(ov_time_total(&state->vf, -1)));
    }
  
  state->cutpoint_begin = 0;
  vorbis_synthesis_init(state->vd,state->vi);
  vorbis_block_init(state->vd,state->vb);
  
  srand(time(NULL));
  
  return state;
}

void oggsplit(char *filename, splt_state *sp_state, double
	      sec_begin, double sec_end, short seekable, 
	      short adjust, float threshold, int *error)
{
  //we do the next split
  sp_state->split.current_split++;
  
  ogg_state *state = sp_state->ostate;
  ogg_stream_state stream_out;
  ogg_packet header_comm;
  ogg_int64_t begin,end = 0, cutpoint = 0;
  
  begin = (ogg_int64_t) (sec_begin * state->vi->rate);
  
  if (sec_end != -1.f)
    {
      if (sec_begin >= sec_end)
	{
	  *error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
	  return;
	}
      if (adjust) {
	if (sec_end != -1) {
	  float gap = (float) adjust;
	  if (sec_end > gap)
	    sec_end -= gap;
	  if (sec_end < sec_begin)
	    sec_end = sec_begin;
	}
	else adjust = 0;
      }
      end = (ogg_int64_t) (sec_end * state->vi->rate);
      cutpoint = end - begin;
    }
  
  if (state->end == 0) {					// First time we run this, no packets already saved.
    if(find_begin_cutpoint(state, state->in, begin)) 	// We must do this before. If an error occurs, we don't want to create empty files!
      {
	*error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
	return;
      }
  }

  if (strcmp(filename, "-")==0)
    {
      state->out = stdout;
    }
  else
    {
      if (!(state->out=fopen(filename, "wb")))
	{
	  *error = MP3SPLT_ERROR_CANNOT_OPEN_DEST_FILE;
	  return;
	}
    }
  
  ogg_stream_init(&stream_out, rand()); /* gets random serial number*/
  
  vorbis_commentheader_out(&state->vc, &header_comm);
  if (state->headers[1])
    free_packet(state->headers[1]);
  state->headers[1] = save_packet(&header_comm);
  //we clear the packet
  ogg_packet_clear(&header_comm);
  vorbis_comment_clear(&state->vc);
  
  submit_headers_to_stream(&stream_out, state);
  
  if(write_pages_to_file(&stream_out, state->out, 1))
    {
      *error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
      ogg_stream_clear(&stream_out);
      return;
    }
  
  if(find_end_cutpoint(sp_state, &stream_out, state->in, state->out, cutpoint, adjust, threshold))
    {
      *error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
      ogg_stream_clear(&stream_out);
      return;
    }
  
  ogg_stream_clear(&stream_out);
  
  fclose(state->out);
  
  if (state->end == -1) 
    {
      *error = MP3SPLT_OK_SPLITTED_OGG_EOF;
      return;
    }
  
  *error = MP3SPLT_OK_SPLITTED_OGG;
}

int ogg_silence(ogg_state *state, vorbis_dsp_state *vd, float threshold)
{
  float **pcm, sample;
  int samples, silence = 1;
  
  while((samples=vorbis_synthesis_pcmout(vd,&pcm))>0)
    {
      if (silence) {
	int i, j;
	for (i=0; i < state->vi->channels; i++) {
	  float  *mono=pcm[i];
	  if (!silence) break;
	  for(j=0; j<samples; j++)
	    {
	      sample = fabs(mono[j]);
	      state->temp_level = state->temp_level *0.999 + sample*0.001;
	      if (sample > threshold)
		silence = 0;
	    }
	}
      }
      vorbis_synthesis_read(vd, samples);
    }
  return silence;
}

int ogg_scan_silence (splt_state *sp_state, short seconds,
		      float threshold, float min, short output, 
		      ogg_page *page, ogg_int64_t granpos)
{
  put_progress_text(sp_state,MP3SPLT_PROGRESS_SCAN_SILENCE);
  
  ogg_state *state = sp_state->ostate;
  ogg_page og;
  ogg_packet op;
  ogg_sync_state oy;
  ogg_stream_state os;
  vorbis_dsp_state vd;
  vorbis_block vb;
  ogg_int64_t end_position, begin_position, pos, end, begin, page_granpos;
  int eos=0, found = 0, shot, result = 0, len = 0 ;
  short first, flush = 0;
  off_t position = ftello(state->in); // Some backups
  int saveW = state->prevW;
  float th = convertfromdB(threshold);
  
  ogg_sync_init(&oy);
  ogg_stream_init(&os, state->serial);
  
  if (page){ // We still have a page to process
    memcpy(&og, page, sizeof(ogg_page));
    result = 1;
  }
  
  end_position = begin_position = pos = granpos;
  vorbis_synthesis_init(&vd, state->vi);
  vorbis_block_init(&vd, &vb);

  if (seconds > 0)
    end = (ogg_int64_t) (seconds * state->vi->rate);
  else end = 0;

  begin = 0;
  first = output;
  shot = MP3SPLT_DEFAULTSHOT;

  state->temp_level = 0.0;
  state->avg_level = 0.0;
  state->n_stat = 0.0;
  
  while (!eos)
    {
      while(!eos)
	{
	  if (result==0) break;
	  if(result>0)
	    {
	      if (ogg_page_eos(&og)) eos=1;
	      page_granpos = ogg_page_granulepos(&og) - state->cutpoint_begin;
	      if (pos == 0) pos = page_granpos;
	      ogg_stream_pagein(&os, &og);
	      while(1)
		{
		  result=ogg_stream_packetout(&os, &op);
		  if(result==0) break; /* need more data */
		  if(result>0)
		    {
		      int bs = get_blocksize(state, state->vi, &op);
		      pos += bs;		      
		      if (pos > page_granpos)
			pos = page_granpos;
		      begin += bs;
		      if(vorbis_synthesis(&vb, &op)==0) {
			vorbis_synthesis_blockin(&vd, &vb);
			if ((!flush) && (ogg_silence(state, &vd, th))) {
			  if (len == 0) begin_position = pos;
			  if (first == 0) len++;
			  if (shot < MP3SPLT_DEFAULTSHOT)
			    shot+=2;
			  end_position = pos;
			}
			else {
			  if (len > MP3SPLT_DEFAULTSILLEN) {
			    if ((flush) || (shot <= 0))
			      {
				float b_position, e_position;
				double temp;
				temp = (double) begin_position;
				temp /= state->vi->rate;
				b_position = (float) temp;
				temp = (double) end_position;
				temp /= state->vi->rate;
				e_position = (float) temp;
				if ((e_position - b_position - min) >= 0.f)
				  {
				    ssplit_new(&state->silence_list, b_position, e_position, len);
				    found++;
				  }
				len = 0;
				shot = MP3SPLT_DEFAULTSHOT;
			      }
			  } else len = 0;
			  if (flush) {
			    eos = 1;
			    break;
			  }
			  if ((first) && (shot <= 0))
			    first = 0;
			  if (shot > 0) shot--;
			}
		      }
		    }
		  if (end)
		    if (begin > end)
		      flush = 1;
		  if (found >= MP3SPLT_MAXTRACKS) eos = 1;
		}
	    }
	  result=ogg_sync_pageout(&oy, &og);
	}
      if(!eos){
	if(update_sync(&oy, state->in)==0)
	  {
	    eos=1;
	  }
	result=ogg_sync_pageout(&oy, &og);
	
	if (sp_state->options.split_mode == 
	    MP3SPLT_OPTION_SILENCE_MODE)
	  {
	    //if we have cancelled the split
	    if (sp_state->cancel_split)
	      {
		eos = 1;
	      }
	    update_progress(sp_state,(float)pos,
			    (float)(state->len),
			    1,0,MP3SPLT_DEFAULT_PROGRESS_RATE2);
	  }
	else
	  {
	    update_progress(sp_state,(float)begin,
			    (float)end,
			    2,0.5,MP3SPLT_DEFAULT_PROGRESS_RATE2);
	  }
      }
    }
  
  ogg_stream_clear(&os);

  vorbis_block_clear(&vb);
  vorbis_dsp_clear(&vd);
  ogg_sync_clear(&oy);

  state->prevW = saveW;
  fseeko(state->in, position, SEEK_SET);

  return found;
}

ogg_state *v_new(void)
{
  ogg_state *s;

  if ((s = malloc(sizeof(ogg_state)))==NULL)
    return NULL;
  
  memset(s, 0, sizeof(ogg_state));
  
  if ((s->sync_in = malloc(sizeof(ogg_sync_state)))==NULL)
    return NULL;
  if ((s->stream_in = malloc(sizeof(ogg_stream_state)))==NULL)
    return NULL;
  if ((s->vd = malloc(sizeof(vorbis_dsp_state)))==NULL)
    return NULL;
  if ((s->vi = malloc(sizeof(vorbis_info)))==NULL)
    return NULL;
  if ((s->vb = malloc(sizeof(vorbis_block)))==NULL)
    return NULL;

  if ((s->headers = malloc(sizeof(v_packet)*3))==NULL)
    return NULL;
  memset(s->headers, 0, sizeof(v_packet)*3);
	
  if ((s->packets = malloc(sizeof(v_packet)*2))==NULL)
    return NULL;
  memset(s->packets, 0, sizeof(v_packet)*2);
	
  return s;
}

/* Full cleanup of internal state and vorbis/ogg structures */
void v_free(ogg_state *s)
{
  if(s)
    {
      if(s->packets)
	{
	  if(s->packets[0])
	    free_packet(s->packets[0]);
	  if(s->packets[1])
	    free_packet(s->packets[1]);
	  free(s->packets);
	}
      if(s->headers)
	{
	  int i;
	  for(i=0; i < 3; i++)
	    if(s->headers[i])
	      free_packet(s->headers[i]);
	  free(s->headers);
	}
      vorbis_comment_clear(&s->vc);
      if(s->vb)
	{
	  vorbis_block_clear(s->vb);
	  free(s->vb);
	}
      if(s->vd)
	{
	  vorbis_dsp_clear(s->vd);
	  free(s->vd);
	}
      if(s->stream_in)
	{
	  ogg_stream_clear(s->stream_in);
	  free(s->stream_in);
	}
      if(s->sync_in)
	{
	  ogg_sync_clear(s->sync_in);
	  free(s->sync_in);
	}
      if (s->vi)
	{
	  vorbis_info_clear(s->vi);
	  free(s->vi);
	}
      ssplit_free(&s->silence_list);
      free(s);
    }
}

static char *checkutf(char *s) // For the moment we only omit invalid character
{
  int i, j=0;
  for (i=0; i < strlen(s); i++) {
    if (s[i]<0x7F)
      s[j++] = s[i];
  }
  s[j] = '\0';
  return s;
}

vorbis_comment *v_comment (vorbis_comment *vc, char *artist, char *album, char *title, 
			   char *tracknum, char *date, char *genre, char *comment)
{
  if (title!=NULL)
    vorbis_comment_add_tag(vc, "title",
			   checkutf(title));
  if (artist!=NULL)
    vorbis_comment_add_tag(vc, "artist",
			   checkutf(artist));
  if (album!=NULL)
    vorbis_comment_add_tag(vc, "album",
			   checkutf(album));
  if (date!=NULL)
    if (strlen(date)>0)
      vorbis_comment_add_tag(vc, "date", date);
  if (genre!=NULL)
    vorbis_comment_add_tag(vc, "genre", genre);
  if (tracknum!=NULL)
    vorbis_comment_add_tag(vc, "tracknumber", tracknum);
  if (comment!=NULL)
    vorbis_comment_add_tag(vc, "comment", comment);
	
  return vc;
}

//frees the ogg_state structure,
//used in the splt_state_free() function
void ogg_state_free (ogg_state *s)
{
  ov_clear(&s->vf);
  v_free(s);
}

#endif
