/**********************************************************
 *
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 * http://mp3splt.sourceforge.net
 *
 *********************************************************/

/**********************************************************
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *
 *********************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "mp3splt_utils.h"

#if !HAVE_FSEEKO
#define fseeko fseek
#define ftello ftell
#endif

//converts string s in hundredth
//and returns seconds
//returns -1 if it cannot convert (usually we hope it can :)
long c_hundreths (char *s)
{
  long minutes=0, seconds=0, hundredths=0, i;
  long hun;
  
  for(i=0; i<strlen(s); i++) // Some checking
    if ((s[i]<0x30 || s[i] > 0x39) && (s[i]!='.'))
      return -1;
  
  if (sscanf(s, "%ld.%ld.%ld", &minutes, &seconds, &hundredths)<2)
    return -1;
  
  if ((minutes < 0) || (seconds < 0) || (hundredths < 0))
    return -1;

  if ((seconds > 59) || (hundredths > 99))
    return -1;
  
  if (s[strlen(s)-2]=='.')
    hundredths *= 10;
  
  hun = hundredths;
  hun += (minutes*60 + seconds) * 100;
  
  return hun;
}

//returns -1 is the file is damaged
//0 otherwise
int getword (FILE *in, off_t offset, int mode, 
	     unsigned long *headw)
{
  int i;
  *headw = 0;

  if (fseeko(in, offset, mode)==-1)
    return -1;
  for (i=0; i<4; i++)
    {
      if (feof(in)) return -1;
      *headw = *headw << 8;
      *headw |= fgetc(in);
    }
  return 0;
}

//returns the file length
off_t flength (FILE *in)
{
  struct stat info;
  if (fstat(fileno(in), &info)==-1)
    {
      return -1;
    }
  return info.st_size;
}

//used for the mp3 sync errors split and mp3 split(for header)
//returns 0 if no errors, MP3SPLT_ defined errors if ones
//It justs copies the data of the input file from a begin offset
//to an end offset, and, eventually, a Xing frame (for VBR)
//at the beginning and a ID3v1 at the end, to an outputfile.
int split (splt_state *state, char *filename,
	   off_t begin, off_t end, char *id3buffer)
{
  FILE *file_output;
  off_t position;
  unsigned char buffer[MP3SPLT_READBSIZE];
  long readed;
  //for the progress
  off_t temp_end = 0;
  
  //the start point of the split
  long start = begin;
  
  put_progress_text(state,MP3SPLT_PROGRESS_CREATE);
  
  if (end != -1)
    if (begin >= end) 
      {
	return MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
      }
  
  position = ftello(state->mstate->file_input); // Save current position
  
  if (fseeko(state->mstate->file_input, begin, SEEK_SET)==-1)
    return MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
  
  // - means stdout huh)
  if (strcmp(filename, "-")==0)
    {
      file_output = stdout;
    }
  else
    {
      if (!(file_output=fopen(filename, "wb+")))
	{
	  return MP3SPLT_ERROR_CANNOT_OPEN_FILE;
	}
    }
  
  if (state->mstate->mp3file.xing!=0)
    {
      if(fwrite(state->mstate->mp3file.xingbuffer, 1, 
		state->mstate->mp3file.xing, file_output) <= 0)
	{
	  return MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
	}
    }
  
  while (!feof(state->mstate->file_input))
    {
      readed = MP3SPLT_READBSIZE;
      if (end!=-1)
	{
	  if (begin>=end) break;
	  if ((end-begin) < MP3SPLT_READBSIZE)
	    readed = end-begin;
	}
      
      if ((readed = fread(buffer, 1, readed, 
			  state->mstate->file_input))==-1)
	break;
    
      if (fwrite(buffer, 1, readed, file_output)==-1)
	{
	  return MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
	}
      begin+=readed;
    
      //we update the progress bar
      if ((state->options.split_mode == MP3SPLT_OPTION_WRAP_MODE) ||
	  (state->options.split_mode == MP3SPLT_OPTION_MP3_ERROR_MODE) ||
	  ((state->options.split_mode == MP3SPLT_OPTION_NORMAL_MODE)
	   && (!state->options.option_auto_adjust) &&
	   (!state->options.option_mp3_frame_mode)))
	{
	  temp_end = end;
	  //for the last split
	  if (end == -1)
	    {
	      temp_end = state->mstate->end2;
	    }
	
	  update_progress(state,(float)(begin-start),
			  (float)(temp_end-start),1,0,
			  MP3SPLT_DEFAULT_PROGRESS_RATE);
	}
      else
	{
	  //if auto adjust, we have 50%
	  if (state->options.option_auto_adjust)
	    {
	      update_progress(state,(float)(begin-start),
			      (float)(end-start),
			      2,0.5,
			      MP3SPLT_DEFAULT_PROGRESS_RATE);
	    }
	  else
	    {
	      if (state->options.split_mode == 
		  MP3SPLT_OPTION_TIME_MODE)
		{
		  temp_end = end;
		  //for the last split
		  if (end == -1)
		    {
		      temp_end = state->mstate->end2;
		    }
		
		  //if framemode
		  if (state->options.option_mp3_frame_mode)
		    {
		      update_progress(state,(float)(begin-start),
				      (float)(temp_end-start),
				      2,0.5,
				      MP3SPLT_DEFAULT_PROGRESS_RATE);
		    }
		  else
		    {
		      update_progress(state,(float)(begin-start),
				      (float)(temp_end-start),
				      1,0,
				      MP3SPLT_DEFAULT_PROGRESS_RATE);
		    }
		}
	      else
		{
		  update_progress(state,(float)(begin-start),
				  (float)(end-start),
				  2,0.5,
				  MP3SPLT_DEFAULT_PROGRESS_RATE);
		}
	    }
	}
    }
  
  if (id3buffer!=NULL)
    {
      if (fseeko(file_output, getid3v1(file_output), SEEK_END)!=-1)
	{
	  if (fwrite(id3buffer, 1, 128, file_output) <= 0)
	    {
	      return MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
	    }
	}
      else
	{
	  return MP3SPLT_ERROR_SEEKING_FILE;
	}
    }
  
  fclose(file_output);
  
  if (fseeko(state->mstate->file_input, position, SEEK_SET)==-1)
    {
      return MP3SPLT_ERROR_SEEKING_FILE;
    }
  
  //if (begin!=end)
  //return MP3SPLT_OK_SPLITTED_MP3;
  
  return MP3SPLT_OK_SPLITTED_MP3;
}

//used for ogg
char *trackstring(int number)
{
  char *track = NULL;
  if (number != 0)
    {
      int len = 0, i;
      len = ((int) (log10((double) (number)))) + 1;
      if ((track = malloc(len + 1))==NULL)
	return NULL;
      memset(track, 0, len + 1);
      for (i=len-1; i >= 0; i--)
	{
	  track[i] = ((number%10) | 0x30);
	  number /= 10;
	}
    }
  return track;
}

//order the splitpoints (used in the silence split)
void order_splitpoints(long *ssplitpoints, int len)
{
  int i, j;
  float key;
  for (j=1; j < len; j++)
    {
      key = ssplitpoints[j];
      i = j -1;
      while ((i >= 0) && (ssplitpoints[i] > key))
	{
	  ssplitpoints[i+1] = ssplitpoints[i];
	  i--;
	}
      ssplitpoints[i+1] = key;
    }
}

//get silence position depending of the offset
float silence_position(struct ssplit *temp, float off)
{
  float position = (temp->end_position - 
		    temp->begin_position);
  position = temp->begin_position + (position*off);
  return position;
}

//free the ssplit structure
void ssplit_free (struct ssplit **silence_list)
{
  struct ssplit *temp, *saved;
  temp = *silence_list;
  while (temp != NULL)
    {
      saved = temp->next;
      free(temp);
      temp = saved;
    }
  *silence_list = NULL;
}

//creates a new ssplit structure
int ssplit_new(struct ssplit **silence_list, float begin_position, float end_position, int len)
{
  struct ssplit *temp;
  struct ssplit *s_new;
  
  if ((s_new = malloc(sizeof(struct ssplit)))==NULL)
    return -1;
  s_new->len = len;
  s_new->begin_position = begin_position;
  s_new->end_position = end_position;
  s_new->next = NULL;

  temp = *silence_list;
  if (temp == NULL)
    {
      *silence_list = s_new; // No elements
    }
  else 
    {
      if (temp->len < len)
	{
	  s_new->next = temp;
	  *silence_list = s_new;
	}
      else  
	{
	  if (temp->next == NULL)
	    temp->next = s_new;
	  else 
	    {
	      while (temp != NULL) 
		{
		  if (temp->next != NULL) 
		    {
		      if (temp->next->len < len) 
			{
			  // We build an ordered list by len to keep most probable silence points
			  s_new->next = temp->next;
			  temp->next = s_new;
			  break;
			}
		    }
		  else 
		    {
		      temp->next = s_new;
		      break;
		    }
		  temp = temp->next;
		}
	    }
	}
    }

  return 0;
}

//cleans the string of weird characters like ? " | : > < * \ \r
char *cleanstring (char *s)
{
  int i, j=0;
  char *copy;
  copy = strdup(s);
  for (i=0; i<=strlen(copy); i++)
    {
      if ((copy[i]!='\\')&&(copy[i]!='/')&&(copy[i]!='?')
	  &&(copy[i]!='*')&&(copy[i]!=':')&&(copy[i]!='"')
	  &&(copy[i]!='>')&&(copy[i]!='<')&&(copy[i]!='|')
	  &&(copy[i]!='\r'))
	{
	  s[j++] = copy[i];
	}
    }
  free(copy);
  // Trim string. I will never stop to be surprised about cddb strings dirtiness! ;-)
  for (i=strlen(s)-1; i >= 0; i--) 
    {
      if (s[i]==' ')
	{
	  s[i] = '\0';
	}
      else 
	break;
    }
  return s;
}

//convert to decibels
float convert2dB(double input)
{
  float level;
  if (input<=0.0)
    level = -96.0;
  else level = 20 * log10(input);
  return level;
}

//convert from decibels
double convertfromdB(float input)
{
  double amp;
  if (input<-96.0)
    amp = 0.0;
  else amp = pow(10.0, input/20.0);
  return amp;
}
