/**********************************************************
 *
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 * http://mp3splt.sourceforge.net
 *
 *********************************************************/

/**********************************************************
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *
 *********************************************************/

#include <sys/stat.h>
#include <string.h>

#include "mp3splt_utils.h"

//returns the filename of the new file to be created, the two integer
//parameters are for the 2 splitpoints
//Warning !! the first splitpoint begins at 0 !!!
//i is the current split, to find the current name in 
//state->fn[i]
//error is the possible error
char *get_new_filename(char *filename, 
		       splt_state *state,
		       long split_begin,
		       long split_end,
		       int i, int *error)
{
  //calculating minutes and seconds to modify filename
  char mins_char[16], secs_char[16], hundr_secs_char[16],
    mins_char2[16], secs_char2[16], hundr_secs_char2[16];
  
  char *fname = NULL, *fname2 = NULL;
  int fname2_malloc_number = 0,fname_malloc_number = 0;
  
  fname2_malloc_number = fname_malloc_number = 
    strlen(filename) + 256;
  
  if((fname = calloc(fname_malloc_number,1)) != NULL)
    {
      if((fname2 = calloc(fname2_malloc_number,1)) != NULL)
	{
	  //hundreds of seconds
	  snprintf(hundr_secs_char,16, "%ld", split_begin % 100);
	  snprintf(hundr_secs_char2,16, "%ld", split_end % 100);
	  //transform to seconds
	  split_begin = split_begin / 100;
	  split_end = split_end / 100;
	  //calculating seconds
	  snprintf(secs_char,16, "%ld", split_begin % 60);
	  snprintf(secs_char2,16, "%ld", split_end % 60);
	  //calculating minutes
	  snprintf(mins_char,16, "%ld", split_begin / 60);
	  snprintf(mins_char2,16, "%ld", split_end / 60);
	  
	  char temp[3];
	  //transform " " to "\ "
	  int j;
	  for (j = 0; j < strlen((char *)state->split.fn[i]); j++)
	    {
	      if ((state->split.fn[i][j] == ' '))
		{
		  strcat(fname, " ");
		}
	      else
		if ((state->split.fn[i][j] == '\\') ||
		    (state->split.fn[i][j] == '/'))
		  {
		    strcat(fname, "-");
		  }
		else
		  {
		    snprintf(temp,2,"%c", state->split.fn[i][j]);
		    strcat(fname,temp);
		  }
	    }
	  
	  //if fn[i] is "", we put the same name as the
	  //original file
	  if (strcmp(state->split.fn[i],"") == 0)
	    {
	      //we cut the extension of the original file
	      snprintf(fname,strlen(filename)-3,"%s",
		       filename);
	    }

	  //final filename of the file
	  //if we have the option set, put seconds and minutes..
	  if (state->options.option_mins_secs)
	    {
	      snprintf(fname2,fname2_malloc_number,
		       "%s_%sm_%ss_%sh__%sm_%ss_%sh", 
		       fname, mins_char, secs_char, hundr_secs_char,
		       mins_char2, secs_char2,hundr_secs_char2);
	    }
	  else
	    {
	      snprintf(fname2,fname_malloc_number,
		       "%s", fname);
	    }
	  
	  //put the extension according to the file type
	  if (state->file_format == MP3SPLT_MP3_FORMAT)
	    {
	      strcat(fname2, MP3SPLT_MP3EXT);
	    }
#ifndef NO_OGG
	  else 
	    {
	      strcat(fname2, MP3SPLT_OGGEXT);
	    }
#endif
  	}
      else
	{
	  *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
	}
      //freeing memory
      free(fname);
    }
  else
    {
      *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
    }
  
  return fname2;
}

//returns the complete filename with the correct path
//error is the possible error
char *get_complete_new_filename(char *filename, splt_state *state,
				int split_begin, int split_end,
				char *new_filename_path,
				int i, int *error)
{
  //filename of the new created file
  char *fname = NULL;
  //filename with the complete path where we are gonna split the file
  char *fname_with_path = NULL;
  
  //get the filename without the path
  filename = get_real_name(filename);
  fname = get_new_filename(filename, state,
			   split_begin, split_end,
			   i,error);
  
  //if no error
  if (*error >= 0)
    {
      int malloc_number = strlen(fname)+
	strlen(new_filename_path)+10;
      
      //if we can allocate memory
      if((fname_with_path = malloc(malloc_number))
	 != NULL)
	{
	  //if the new filename path is not empty
	  if (strcmp(new_filename_path, "") != 0)
	    {
	      //construct full filename with path
	      snprintf(fname_with_path, malloc_number,
		       "%s%c%s",new_filename_path, 
		       MP3SPLT_DIRCHAR, fname);
	    }
	  else
	    {
	      //construct full filename without path(the current dir)
	      snprintf(fname_with_path, malloc_number,
		       "%s",  fname);
	    }
	}
      else
	{
	  *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
	}
      
      //freeing memory
      free(fname);
    }
  else
    //if error return NULL
    {
      return NULL;
    }
  
  return fname_with_path;
}

//parse the output format to see if correct
int parse_outformat(char *s, splt_state *state)
{
  char *ptrs, *ptre;
  int i=0, amb=MP3SPLT_OUTPUT_FORMAT_AMBIGUOUS, len=0;

  for (i=0; i<strlen(s); i++)
    {
      if (s[i]=='+') 
	{
	  s[i]=' ';
	}
      else 
	{
	  if (s[i]==MP3SPLT_VARCHAR) 
	    {
	      s[i]='%';
	    }
	}
    }
  
  cleanstring(s);
  ptrs = s;
  i=0;
  ptre=strchr(ptrs+1, '%');
  if (s[0]!='%')
    {
      if (ptre==NULL)
	{
	  len=strlen(ptrs);
	}
      else
	{
	  len = ptre-ptrs;
	}
      if (len > MP3SPLT_MAXOLEN)
	{
	  len = MP3SPLT_MAXOLEN;
	}
      strncpy(state->oformat.format[i++], ptrs, len);
    }
  else
    {
      ptre=s;
    }
  
  if (ptre==NULL)
    {
      return MP3SPLT_OUTPUT_FORMAT_AMBIGUOUS;
    }
  ptrs = ptre;
  
  while (((ptre=strchr(ptrs+1, '%'))!=NULL) && 
	 (i < MP3SPLT_OUTNUM))
    {
      char cf = *(ptrs+1);
      
      len = ptre-ptrs;
      if (len > MP3SPLT_MAXOLEN)
	{
	  len = MP3SPLT_MAXOLEN;
	}
      
      switch (cf) {
      case 'a':
	break;
      case 'b':
	break;
      case 't':
	break;
      case 'n':
	amb = MP3SPLT_OUTPUT_FORMAT_OK;
	break;
      case 'p':
	break;
      default:
	return MP3SPLT_OUTPUT_FORMAT_ERROR;
      }
      strncpy(state->oformat.format[i++], ptrs, len);
      ptrs = ptre;
    }
  
  strncpy(state->oformat.format[i], ptrs, strlen(ptrs));

  if (ptrs[1]=='t')
    amb = MP3SPLT_OUTPUT_FORMAT_OK;

  if (ptrs[1]=='n')
    amb = MP3SPLT_OUTPUT_FORMAT_OK;
  
  free(s);
  
  return amb;
}

//convert to float for hundredth
// 3460 -> 34.6  | 34 seconds and 6 hundredth
double get_double_pos(long split)
{
  double pos;
  pos = split / 100;
  pos += ((split % 100) / 100);
  return pos;
}

//cleans one split data
void clean_one_split_data(splt_state *state, int num)
{
  memset(state->tags.id[num].year,'\0',31);
  memset(state->tags.id[num].artist,'\0',31);
  memset(state->tags.id[num].album,'\0',31);
  memset(state->tags.id[num].title,'\0',128);
  memset(state->tags.id[num].comment,'\0',31);
  memset(state->tags.performer[num],'\0',128);
  memset(state->split.fn[num],'\0',256);
}

//cleans data for the strings in *state
void clean_split_data(splt_state *state,int tracks)
{
  state->split.current_split = 0;
  do {
    clean_one_split_data(state,state->split.current_split);
    state->split.current_split++;
  } while (state->split.current_split<=tracks);
}

//cleans original id3 tags
void clean_original_id3(splt_state *state)
{
  memset(state->original_tags.year,'\0',31);
  memset(state->original_tags.artist,'\0',31);
  memset(state->original_tags.album,'\0',31);
  memset(state->original_tags.title,'\0',128);
  memset(state->original_tags.comment,'\0',31);
  state->original_tags.track = -1;
  //12 means "other"
  state->original_tags.genre = 12;
}

//writes the filenames according to the output_filename
void put_output_filename(splt_state *state)
{
  char temp[256], fm[256];
  int i;
  char output_filename[512];
  memset(output_filename, 0, 512);
  for (i=0; i<MP3SPLT_OUTNUM; i++)
    {
      memset(temp, 0, 256);
      memset(fm, 0, 256);
      if (strlen(state->oformat.format[i])==0)
	break;
      if (state->oformat.format[i][0]=='%')
	{
	  temp[0]='%';
	  temp[1]='s';
	  switch (state->oformat.format[i][1])
	    {
	    case 'a':
	      //if (cddboption)
	      {
		sprintf(temp+2, state->oformat.format[i]+2);
		if (state->options.option_output_is_default)
		  {
		    sprintf(fm, temp, state->tags.id[state->split.current_split].artist);
		  }
		else
		  {
		    sprintf(fm, temp, state->tags.id[0].artist);
		  }
	      }
	      break;
	    case 'b':
	      //if (cddboption)
	      {
		sprintf(temp+2, state->oformat.format[i]+2);
		sprintf(fm, temp, state->tags.id[0].album);
	      }
	      break;
	    case 't':
	      //if (cddboption)
	      {
		sprintf(temp+2, state->oformat.format[i]+2);
		sprintf(fm, temp, state->tags.id[state->split.current_split].title);
	      }
	      break;
	    case 'p':
	      //if (cddboption)
	      {
		sprintf(temp+2, state->oformat.format[i]+2);
		sprintf(fm, temp, state->tags.performer[state->split.current_split]);
	      }
	      break;
	    case 'n':
	      temp[1]='0';
	      temp[2]=state->oformat.output_format_digits;
	      temp[3]='d';
	      sprintf(temp+4, state->oformat.format[i]+2);
	      sprintf(fm, temp, state->split.current_split+1);
	      break;
	    }
	}
      else 
	{
	  strncpy(fm, state->oformat.format[i], MP3SPLT_MAXOLEN);
	}
      
      strncat(output_filename, fm, strlen(fm));
    }
  
  snprintf(state->split.fn[state->split.current_split],255,
	   output_filename);
}

//finding the real name of the file, without the path
char *get_real_name(char *filename)
{
  char *c = NULL;
  while ((c = strchr(filename, MP3SPLT_DIRCHAR))
	 !=NULL)
    filename = c + 1;
  
  return filename;
}

//puts in the state->progress_text the real text 
//of the progress
//type = 0 we put " preparing... "
//type = 1, we put " creating ... ",
//type = 2 we put " searching for sync errors ..."
//type = 3 we put " scanning for silence ..."
//we write the filename that we are currently splitting
void put_progress_text(splt_state *state,int type)
{
  //filename_shorted_length = sp_state->progress_text_max_char;
  //if we have a too long filename, display it shortly
  char filename_shorted[512];
  
  //if we have a progress function
  if (state->split.p_bar.progress != NULL)
    {
      snprintf(filename_shorted,
	       state->split.p_bar.progress_text_max_char,"%s",
	       state->split.fn[state->split.current_split-1]);
      if (strlen(state->split.fn[state->split.current_split-1])
	  > state->split.p_bar.progress_text_max_char)
	{
	  filename_shorted[strlen(filename_shorted)-1] = '.';
	  filename_shorted[strlen(filename_shorted)-1] = '.';
	}
      
      //we put the extension of the song
      char extension[10] = MP3SPLT_MP3EXT;
      if (state->file_format == MP3SPLT_OGG_FORMAT)
	{
	  snprintf(extension, 9, MP3SPLT_OGGEXT);
	}
      
      snprintf(state->split.p_bar.filename_shorted, 511,"%s%s",
	       filename_shorted, extension);
      
      state->split.p_bar.progress_type = type;
    }
}

//update the progress,
//current_point = the current progress point
//total_points = total progress points
//split_stage = 1 means we put on the whole progress bar,
//if split_stage = 2,
//progress_start = from where to start the progress (fraction)
//refresh_rate = the refresh rate of the display
void update_progress(splt_state *state, float current_point,
		     float total_points, int progress_stage,
		     float progress_start, int refresh_rate)
{
  //if we have a progress callback function
  if (state->split.p_bar.progress != NULL)
    {
      //shows the progress
      state->split.p_bar.percent_progress = 
	(current_point / total_points);
      
      state->split.p_bar.percent_progress = 
	state->split.p_bar.percent_progress / progress_stage +
	progress_start;
      
      //security check
      if (state->split.p_bar.percent_progress < 0)
	{
	  state->split.p_bar.percent_progress = 0;
	}
      if (state->split.p_bar.percent_progress > 1)
	{
	  state->split.p_bar.percent_progress = 1;
	}
      
      //call
      state->split.p_bar.progress(refresh_rate);
    }
}

//says to the program using the library the splitted file
void put_splitted_file(splt_state *state,
		       char *filename)
{
  //put splitted file 
  if (state->split.file_splitted != NULL)
    {
      state->split.file_splitted(filename);
    }
}

//check if its a file
int is_file(char *fname)
{
  struct stat buffer;
  int         status;
  
  status = stat(fname, &buffer);
  //if it is a file
  if (S_ISREG(buffer.st_mode) != 0)
    return MP3SPLT_TRUE;
  else
    return MP3SPLT_FALSE;
}

/* structures initialisation and frees */
void free_freedb_search(freedb_results *search_results)
{
  if (search_results != NULL)
    {
      if (search_results->number != 0)
	{
	  int i;
	  for(i = 0; i < search_results->number;i++)
	    {
	      if (search_results->results[i].revision_number 
		  != 0)
		free(search_results->results[i].revisions);
	      free(search_results->results[i].name);
	    }
	  free(search_results->results);
	}
      //search_results->number = 0;
      free(search_results);
    }
}

//frees the memory for the freedb results of the search
void free_search_results(cd_state *cdstate,
			 freedb_results *search_results)
{  
  free_freedb_search(search_results);
  //freeing the cdstate
  if (cdstate != NULL)
    { 
      free(cdstate);
    }
}

//create new state aux, allocationg memory
//error is the possible error
splt_state *splt_state_new(splt_state *state, int *error)
{
  if ((state =malloc(sizeof(splt_state))) ==NULL)
    {
      *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
    }
  else
    {
      memset(state, 0x0, sizeof(splt_state));
    }
  
  return state;
}

//reset the state
splt_state *splt_state_reset(splt_state *state)
{
  if (state)
    {
      memset(state, 0x0, sizeof(splt_state));
    }
  return state;
}

//free the state
void splt_state_free(splt_state *state)
{
  if (state!=NULL)
    {
      splt_state_reset(state);
      free(state);
    }
}

//frees an array of strings
void free_files(char **files, int number)
{
  int i = 0;
  if (files != NULL)
    {
      if (number != 0)
	{
	  for (i = 0; i < number; i++)
	    {
	      free(files[i]);
	    }
	  free(files);
	}
    }
}

//cuts spaces from the begin
char *cut_spaces_from_begin(char *c)
{
  //cut spaces from the begin
  if (*c == ' ')
    {
      while (*c == ' ')
	{
	  c++;
	}
    }
  
  return c;
}

//cuts spaces at the end
char *cut_spaces_at_the_end(char *c)
{
  //we cut spaces at the end
  while (*c == ' ')
    {
      *c = '\0';
      c--;
    }
  
  return c;
}

//puts the default options for a normal split
void splt_state_put_default_options(splt_state *state)
{
  state->file_format = MP3SPLT_MP3_FORMAT;
  state->fdb.search_results = NULL;
  state->fdb.cdstate = NULL;
  state->wrap.wrap_files = NULL;
  state->wrap.wrap_files_num = 0;
  state->cancel_split = MP3SPLT_FALSE;
  snprintf(state->oformat.format_char,255,"%s",
	   MP3SPLT_DEFAULT_OUTPUT);
  state->split.put_message = NULL;
  state->split.file_splitted = NULL;
  state->split.p_bar.progress = NULL;
  state->split.p_bar.progress_text_max_char = 40;
  state->options.option_output_is_default = MP3SPLT_TRUE;
  state->options.tags = MP3SPLT_TAGS_ORIGINAL_FILE;
  state->options.option_mins_secs = MP3SPLT_FALSE;
  state->options.split_mode = MP3SPLT_OPTION_NORMAL_MODE;
  state->options.option_mp3_frame_mode = MP3SPLT_TRUE;
  state->options.split_time = 6000;
  state->options.option_auto_adjust = MP3SPLT_FALSE;
  state->options.option_input_not_seekable = MP3SPLT_FALSE;
  state->options.parameter_threshold = MP3SPLT_DEFAULT_PARAM_THRESHOLD;
  state->options.parameter_offset = MP3SPLT_DEFAULT_PARAM_OFFSET;
  state->options.parameter_number_tracks = MP3SPLT_DEFAULT_PARAM_TRACKS;
  state->options.parameter_minimum_length = MP3SPLT_DEFAULT_PARAM_MINIMUM_LENGTH;
  state->options.parameter_remove_silence = MP3SPLT_FALSE;
  state->options.parameter_gap = MP3SPLT_DEFAULT_PARAM_GAP;
}

